-- -----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
Entity Nios2_OCDS_Wrapper Is
    Port(
       -- Soft JTAG Lines
       Altium_TCK  : In  Std_Logic;
       Altium_TDI  : In  Std_Logic;
       Altium_TDO  : Out Std_Logic;
       Altium_TMS  : In  Std_Logic;
       Altium_TRST : In  Std_Logic;

       Reset_n     : In Std_Logic;
       -- Processor Debug Interface
        vji_ir_out  : in  std_logic_vector(1 downto 0);
        vji_tdo     : in  std_logic;
        vji_cdr     : out std_logic := '0';
        vji_ir_in   : out std_logic_vector(1 downto 0);
        vji_rti     : out std_logic := '0';
        vji_sdr     : out std_logic := '0';
        vji_tck     : out std_logic := '0';
        vji_tdi     : out std_logic;
        vji_udr     : out std_logic := '0';
        vji_uir     : out std_logic := '0';
        LAX_test_bus    : out std_logic_vector(15 downto 0)
    );
End;
-- -----------------------------------------------------------------------------


-- -----------------------------------------------------------------------------
Architecture RTL Of Nios2_OCDS_Wrapper Is
-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
    -----------------------------------------------------------------------------
    Type TTapState is(
                        TLR,
                        IDLE,
                        DRSELECT,
                        DRCAPTURE,
                        DRSHIFT,
                        DREXIT1,
                        DRPAUSE,
                        DREXIT2,
                        DRUPDATE,
                        IRSELECT,
                        IRCAPTURE,
                        IRSHIFT,
                        IREXIT1,
                        IRPAUSE,
                        IREXIT2,
                        IRUPDATE
                    );
    Signal CurrentTapState              : TTapState := TLR;--!!
    Signal NextTapState                 : TTapState;
    attribute syn_encoding : string;
    attribute syn_encoding of TTapState : type is "one-hot";
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    -- Altium assgined JTAG ID code for NIOS2 is 0x10099473
    Constant CNios2_ID_CODE : Std_Logic_Vector(31 Downto 0) := "0001" & "0000000010011001" & "01000111001" & '1' ;
    Constant cBypass_IR     : Std_Logic_Vector(3 Downto 0) := "1111";
    Constant cBSP_IR        : Std_Logic_Vector(3 Downto 0) := "0000";
    Constant cIdReg_IR      : Std_Logic_Vector(3 Downto 0) := "0001";
    Constant cReset_IR      : Std_Logic_Vector(3 Downto 0) := "0010";
    Constant cOciMem_IR     : Std_Logic_Vector(3 Downto 0) := "1000";
    Constant cTraceMem_IR   : Std_Logic_Vector(3 Downto 0) := "1001";
    Constant cBreak_IR      : Std_Logic_Vector(3 Downto 0) := "1010";
    Constant cTraceCtrl_IR  : Std_Logic_Vector(3 Downto 0) := "1011";
    ----------------------------------------------------------------------------
    Signal IR_Reg                       : Std_Logic_Vector(3 Downto 0);
    Signal IR_Reg_Tmp                   : Std_Logic_Vector(3 Downto 0);
    Signal Bypass                       : Std_Logic;
    Signal ShiftIdCodeReg               : Std_Logic_Vector(31 Downto 0);
    
-- -----------------------------------------------------------------------------
Begin
-- -----------------------------------------------------------------------------
    vji_ir_in   <= IR_Reg_Tmp(1 downto 0);
    --vji_ir_in <= IR_Reg(1 downto 0);

    vji_cdr <= '1' When (CurrentTapState = DRCAPTURE)  Else '0';
    vji_rti <= '1' When (CurrentTapState = IDLE)        Else '0';
    vji_sdr   <= '1' When (CurrentTapState = DRSHIFT)   Else '0';
    vji_udr   <= '1' When (CurrentTapState = DRUPDATE)  Else '0'; 
    vji_uir   <= '1' When (CurrentTapState = IRUPDATE)  Else '0'; 
    vji_tck   <= Altium_TCK;           
    vji_tdi   <= Altium_TDI;
    ----------------------------------------------------------------------------
    -- Tap controller
    ----------------------------------------------------------------------------
    TapController:
    Process ( CurrentTapState,
              Altium_TMS )
    Begin
       Case CurrentTapState Is
          When TLR       => If (Altium_TMS='0') Then NextTapState <= IDLE;      Else NextTapState <= TLR;      End If;--
          When IDLE      => If (Altium_TMS='0') Then NextTapState <= IDLE;      Else NextTapState <= DRSELECT; End if;--
          When DRSELECT  => If (Altium_TMS='0') Then NextTapState <= DRCAPTURE; Else NextTapState <= IRSELECT; End If;--
          When DRCAPTURE => If (Altium_TMS='0') Then NextTapState <= DRSHIFT;   Else NextTapState <= DREXIT1;  End If;--
          When DRSHIFT   => If (Altium_TMS='0') Then NextTapState <= DRSHIFT;   Else NextTapState <= DREXIT1;  End If;--
          When DREXIT1   => If (Altium_TMS='0') Then NextTapState <= DRPAUSE;   Else NextTapState <= DRUPDATE; End If;--
          When DRPAUSE   => If (Altium_TMS='0') Then NextTapState <= DRPAUSE;   Else NextTapState <= DREXIT2;  End If;--
          When DREXIT2   => If (Altium_TMS='0') Then NextTapState <= DRSHIFT;   Else NextTapState <= DRUPDATE; End If;--
          when DRUPDATE  => If (Altium_TMS='0') Then NextTapState <= IDLE;      Else NextTapState <= DRSELECT; End If;--
          When IRSELECT  => If (Altium_TMS='0') Then NextTapState <= IRCAPTURE; Else NextTapState <= TLR;      End If;--
          When IRCAPTURE => If (Altium_TMS='0') Then NextTapState <= IRSHIFT;   Else NextTapState <= IREXIT1;  End If;--
          When IRSHIFT   => If (Altium_TMS='0') Then NextTapState <= IRSHIFT;   Else NextTapState <= IREXIT1;  End If;--
          When IREXIT1   => If (Altium_TMS='0') Then NextTapState <= IRPAUSE;   Else NextTapState <= IRUPDATE; End If;--
          When IRPAUSE   => If (Altium_TMS='0') Then NextTapState <= IRPAUSE;   Else NextTapState <= IREXIT2;  End If;--
          When IREXIT2   => If (Altium_TMS='0') Then NextTapState <= IRSHIFT;   Else NextTapState <= IRUPDATE; End If;--
          When IRUPDATE  => If (Altium_TMS='0') Then NextTapState <= IDLE;      Else NextTapState <= DRSELECT; End If;--
       End Case;
    End Process;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    StateShifter:
    Process(Altium_TCK, Altium_TRST)
    Begin
        If Altium_TRST = '0' Then
            CurrentTapState <= TLR;
        ElsIf Rising_Edge(Altium_TCK) Then
            CurrentTapState <= NextTapState;
        End If;
    End Process;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    RegisterController:
    Process(Altium_TCK)
    Begin
      If Rising_Edge(Altium_TCK) Then
         Bypass <= Altium_TDI;
         
         Case CurrentTapState Is
            When TLR =>
                IR_Reg      <= cIdReg_IR;
                --IR_Reg_Tmp    <= cIdReg_IR;
                ShiftIdCodeReg <= cNios2_Id_Code;
            When IDLE =>
                ShiftIdCodeReg <= cNios2_Id_Code;
            When DRSHIFT =>
                If IR_Reg=cIdReg_IR Then
                    ShiftIdCodeReg <= Altium_TDI & ShiftIdCodeReg(31 Downto 1);
                End If;
            When IRUPDATE =>
                IR_Reg <= IR_Reg_Tmp;
            When IRCAPTURE =>
                IR_Reg <= cIdReg_IR;
                --IR_Reg_Tmp(1 downto 0)    <= B"01";
            When IRSHIFT =>
                IR_Reg_Tmp <= Altium_TDI & IR_Reg_Tmp(3 Downto 1);
            When Others =>
                Null;
         End Case;
      End If;
    End Process;
    ----------------------------------------------------------------------------
        Altium_TDO <=   Bypass      When(CurrentTapState=DRSHIFT And IR_Reg=cBypass_IR) Else
                        IR_Reg_Tmp(0)       When(CurrentTapState=IRSHIFT)                           Else                    
                        ShiftIdCodeReg(0)   When (CurrentTapState=DRSHIFT And IR_Reg=cIdReg_IR)     Else
                        vji_tdo             When (CurrentTapState=DRSHIFT And IR_Reg=cOciMem_IR)    Else
                        vji_tdo             When (CurrentTapState=DRSHIFT And IR_Reg=cTraceMem_IR)  Else
                        vji_tdo             When (CurrentTapState=DRSHIFT And IR_Reg=cBreak_IR)     Else
                        vji_tdo             When (CurrentTapState=DRSHIFT And IR_Reg=cTraceCtrl_IR) Else
                        Bypass;--'0';
    -- -----------------------------------------------------------------------------
    -- -----------------------------------------------------------------------------
    LAX_test_bus(15 downto 9)       <= (others => '0');
    LAX_test_bus(0) <= '1' when (IR_Reg = cBypass_IR)       Else '0';
    LAX_test_bus(1) <= '1' when (IR_Reg = cBSP_IR)          Else '0';
    LAX_test_bus(2) <= '1' when (IR_Reg = cIdReg_IR)        Else '0';
    LAX_test_bus(3) <= '1' when (IR_Reg = cReset_IR)        Else '0';
    LAX_test_bus(4) <= '1' when (IR_Reg = cOciMem_IR)       Else '0';
    LAX_test_bus(5) <= '1' when (IR_Reg = cTraceMem_IR)     Else '0';
    LAX_test_bus(6) <= '1' when (IR_Reg = cBreak_IR)        Else '0';
    LAX_test_bus(7) <= '1' when (IR_Reg = cTraceCtrl_IR)    Else '0';
    LAX_test_bus(8) <= '1' when (CurrentTapState = IRCAPTURE)   Else '0';
-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
End RTL;
-- -----------------------------------------------------------------------------
