------------------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
Use     IEEE.Std_Logic_Unsigned.All;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
Entity Configurable_ReplaceWith_Designator Is
   Port (										-- 2014-09-03
        ----------------------------------------------------------------------------
        CLK_I          : In    Std_Logic;
        RST_I          : In    Std_Logic;
        INT_I          : In    Std_Logic_Vector(31 DownTo 0);
        ----------------------------------------------------------------------------

        ----------------------------------------------------------------------------
        -- Leave these Out if not including OCDS Unit
        TCK            : In    Std_Logic;                         -- IncludeIf_OCDS
        TDI            : In    Std_Logic;                         -- IncludeIf_OCDS
        TDO            : Out   Std_Logic;                         -- IncludeIf_OCDS
        TMS            : In    Std_Logic;                         -- IncludeIf_OCDS
        TRST           : In    Std_Logic;                         -- IncludeIf_OCDS
        ----------------------------------------------------------------------------

        ----------------------------------------------------------------------------
        IO_ACK_I       : In    Std_Logic;
        IO_ADR_O       : Out   Std_Logic_Vector(23 DownTo 0);
        IO_CYC_O       : Out   Std_Logic;
        IO_DAT_I       : In    Std_Logic_Vector(31 DownTo 0);
        IO_DAT_O       : Out   Std_Logic_Vector(31 DownTo 0);
        IO_SEL_O       : Out   Std_Logic_Vector( 3 DownTo 0);
        IO_STB_O       : Out   Std_Logic;
        IO_WE_O        : Out   Std_Logic;
        IO_CLK_O       : Out   Std_Logic;
        IO_RST_O       : Out   Std_Logic;
        ----------------------------------------------------------------------------

        ----------------------------------------------------------------------------
        ME_ACK_I       : In    Std_Logic;
        ME_ADR_O       : Out   Std_Logic_Vector(31 DownTo 0);
        ME_CYC_O       : Out   Std_Logic;
        ME_DAT_I       : In    Std_Logic_Vector(31 DownTo 0);
        ME_DAT_O       : Out   Std_Logic_Vector(31 DownTo 0);
        ME_SEL_O       : Out   Std_Logic_Vector( 3 DownTo 0);
        ME_STB_O       : Out   Std_Logic;
        ME_WE_O        : Out   Std_Logic;
        ME_CLK_O       : Out   Std_Logic;
        ME_RST_O       : Out   Std_Logic;
        ----------------------------------------------------------------------------

        ----------------------------------------------------------------------------
        -- For Debugging
        ----------------------------------------------------------------------------
        LAX_CPU_I_ReadData        : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_I_WaitRequest     : Out Std_Logic;
        LAX_CPU_I_Address         : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_I_Read            : Out Std_Logic;
        LAX_CPU_I_ReadDataValid   : Out Std_Logic;
        LAX_CPU_D_Address         : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_ByteEnable      : Out Std_Logic_Vector(3  Downto 0);
        LAX_CPU_D_Read            : Out Std_Logic;
        LAX_CPU_D_Write           : Out Std_Logic;
        LAX_CPU_D_ReadData        : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_WriteData       : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_WaitRequest     : Out Std_Logic;
		LAX_CPU_D_ReadDataValid   : Out Std_Logic; -- IncludeIf_HasDataCache
		
        LAX_MainMemory_I_Wait     : Out Std_Logic;
        LAX_MainMemory_D_Wait     : Out Std_Logic;
        LAX_Peripheral_I_Wait     : Out Std_Logic;
        LAX_Peripheral_D_Wait     : Out Std_Logic;
        LAX_InternalMem_I_Wait    : Out Std_Logic;
        LAX_InternalMem_D_Wait    : Out Std_Logic;
        LAX_DebugMem_I_Wait       : Out Std_Logic;
        LAX_DebugMem_D_Wait       : Out Std_Logic;
		LAX_TimeBase_I_Wait       : Out Std_Logic;
        LAX_TimeBase_D_Wait       : Out Std_Logic;
		
        LAX_Debug_State_IO                     : Out Std_Logic_Vector(1 Downto 0);
        LAX_Debug_StateNext_IO                 : Out Std_Logic_Vector(1 Downto 0);
        LAX_Debug_Access_IO_Data  			   : Out Std_Logic;
        LAX_Debug_Access_IO_Instr 			   : Out Std_Logic;
		LAX_Debug_Access_ME_Data               : Out Std_Logic;
		LAX_Debug_Access_ME_Instr              : Out Std_Logic;
		LAX_Debug_Access_ME                    : Out Std_Logic;
		LAX_Debug_Access_InternalMem_Data      : Out Std_Logic;
		LAX_Debug_Access_InternalMem_Instr     : Out Std_Logic;
		LAX_Debug_Access_DebugMem_Data         : Out Std_Logic;
		LAX_Debug_Access_DebugMem_Instr        : Out Std_Logic;
		LAX_Debug_Access_TimeBaseMem_Data      : Out Std_Logic;
		LAX_Debug_Access_TimeBaseMem_Instr     : Out Std_Logic;
		
		LAX_jtag_module_debugaccess_to_roms : out std_logic;	 -- IncludeIf_OCDS
		LAX_jtag_module_resetrequest        : out std_logic;	 -- IncludeIf_OCDS
		LAX_jtag_module_debugaccess         : out std_logic;	 -- IncludeIf_OCDS
		LAX_jtag_module_read                : out std_logic;	 -- IncludeIf_OCDS
		LAX_jtag_module_waitrequest         : out std_logic;	 -- IncludeIf_OCDS
		LAX_jtag_module_write               : out std_logic;	 -- IncludeIf_OCDS
		
		LAX_TAP_ir_out                  : out std_logic_vector(1 downto 0);	 -- IncludeIf_OCDS
		LAX_TAP_tdo                     : out std_logic;					 -- IncludeIf_OCDS
		LAX_TAP_cdr                     : out std_logic;					 -- IncludeIf_OCDS
		LAX_TAP_ir_in                   : out std_logic_vector(1 downto 0);	 -- IncludeIf_OCDS
		LAX_TAP_rti                     : out std_logic;					 -- IncludeIf_OCDS
		LAX_TAP_sdr                     : out std_logic;					 -- IncludeIf_OCDS
		LAX_TAP_tck                     : out std_logic;					 -- IncludeIf_OCDS
		LAX_TAP_tdi                     : out std_logic;					 -- IncludeIf_OCDS
		LAX_TAP_udr                     : out std_logic;					 -- IncludeIf_OCDS
		LAX_TAP_uir                     : out std_logic;					 -- IncludeIf_OCDS
		
		LAX_OCDS_test_bus					: out std_logic_vector(15 downto 0);	-- IncludeIf_OCDS
		
		LAX_I_AddressIsInternalMemory : Out Std_Logic;
		LAX_I_AddressIsPeripheral     : Out Std_Logic;
		LAX_I_AddressIsExternalMemory : Out Std_Logic;
		LAX_I_AddressIsDebugMemory    : Out Std_Logic;
		LAX_I_AddressIsTimeBaseMemory : Out Std_Logic;
		LAX_D_AddressIsInternalMemory  : Out Std_Logic;
		LAX_D_AddressIsPeripheral      : Out Std_Logic;
		LAX_D_AddressIsExternalMemory  : Out Std_Logic;
		LAX_D_AddressIsDebugMemory     : Out Std_Logic;
		LAX_D_AddressIsTimeBaseMemory  : Out Std_Logic
    );
End Configurable_ReplaceWith_Designator;
------------------------------------------------------------------------------------


------------------------------------------------------------------------------------
Architecture RTL Of Configurable_ReplaceWith_Designator Is
------------------------------------------------------------------------------------
    Component Nios2_OCDS -- 2014-09-03
        Port (
            Altium_TCK     	: In  Std_Logic;
            Altium_TDI     	: In  Std_Logic;
            Altium_TMS     	: In  Std_Logic;
            Altium_TRST    	: In  Std_Logic;
            Altium_TDO     	: Out Std_Logic;

            Reset_n        	: In  Std_Logic;
			-- Processor Debug Interface
			vji_ir_out		: in  std_logic_vector(1 downto 0);
			vji_tdo			: in  std_logic;
			vji_cdr			: out std_logic;
			vji_ir_in		: out std_logic_vector(1 downto 0);
			vji_rti			: out std_logic;
			vji_sdr     	: out std_logic;
			vji_tck     	: out std_logic;
			vji_tdi     	: out std_logic;
			vji_udr     	: out std_logic;
			vji_uir     	: out std_logic;
			LAX_test_bus	: out std_logic_vector(15 downto 0)	
        );
    End Component;
	
    Component ReplaceWith_Designator_Nios2_Template  -- 2014-09-03
        Port (
 --------------------------------------------------------------------
        CLK_I                    : In  Std_Logic;
        RST_I                    : In  Std_Logic;
        INT_I                    : In  Std_Logic_Vector(31 Downto 0);
        --------------------------------------------------------------------
		-- Peripheral Interface
        --------------------------------------------------------------------
        IO_STB_O                 : Out Std_Logic;
        IO_CYC_O                 : Out Std_Logic;
        IO_ACK_I                 : In  Std_Logic;
        IO_ADR_O                 : Out Std_Logic_Vector(23 Downto 0);
        IO_DAT_I                 : In  Std_Logic_Vector(31 Downto 0);
        IO_DAT_O                 : Out Std_Logic_Vector(31 Downto 0);
        IO_SEL_O                 : Out Std_Logic_Vector(3  Downto 0);
        IO_WE_O                  : Out Std_Logic;
        IO_CLK_O                 : Out Std_Logic;
        IO_RST_O                 : Out Std_Logic;
        --------------------------------------------------------------------
		-- Internal Memory Interface
        --------------------------------------------------------------------
        DataRamIn                : In  Std_Logic_Vector(31 Downto 0);
        DataRamOut               : Out Std_Logic_Vector(31 Downto 0);
        DataRamBE                : Out Std_Logic_Vector(3  Downto 0);
        DataRamWE                : Out Std_Logic;
        DataRamAddr              : Out Std_Logic_Vector(31 Downto 0);
        InstrRomIn               : In  Std_Logic_Vector(31 Downto 0);
        InstrRomAddr             : Out Std_Logic_Vector(31 Downto 0);
        --------------------------------------------------------------------
		-- External Memory Interface
        --------------------------------------------------------------------
        ME_STB_O                 : Out Std_Logic;
        ME_CYC_O                 : Out Std_Logic;
        ME_ACK_I                 : In  Std_Logic;
        ME_ADR_O                 : Out Std_Logic_Vector(31 Downto 0);
        ME_DAT_I                 : In  Std_Logic_Vector(31 Downto 0);
        ME_DAT_O                 : Out Std_Logic_Vector(31 Downto 0);
        ME_SEL_O                 : Out Std_Logic_Vector(3  Downto 0);
        ME_WE_O                  : Out Std_Logic;
        ME_CLK_O                 : Out Std_Logic;
        ME_RST_O                 : Out Std_Logic;
        --------------------------------------------------------------------
		-- TimeBase Memory Interface
        --------------------------------------------------------------------
        TimeBaseIn               	: In  Std_Logic_Vector(31 Downto 0);
        TimeBaseOut              	: Out Std_Logic_Vector(31 Downto 0);
        TimeBaseAddr             	: Out Std_Logic_Vector(1 Downto 0);
        TimeBaseWE               	: Out Std_Logic;
        TimeBaseInterrupt        	: In  Std_Logic;
        TimeBaseReset            	: Out Std_Logic;
		--------------------------------------------------------------------
		-- Virtual JTAG Module Interface
        --------------------------------------------------------------------
		JTAG_ir_out					: out std_logic_vector(1 downto 0);	-- IncludeIf_OCDS
		JTAG_tdo					: out std_logic;					-- IncludeIf_OCDS
		JTAG_cdr					: in  std_logic;					-- IncludeIf_OCDS
		JTAG_ir_in					: in  std_logic_vector(1 downto 0);	-- IncludeIf_OCDS
		JTAG_rti					: in  std_logic;					-- IncludeIf_OCDS
		JTAG_sdr					: in  std_logic;					-- IncludeIf_OCDS
		JTAG_tck					: in  std_logic;					-- IncludeIf_OCDS
		JTAG_tdi					: in  std_logic;					-- IncludeIf_OCDS
		JTAG_udr					: in  std_logic;					-- IncludeIf_OCDS
		JTAG_uir					: in  std_logic;					-- IncludeIf_OCDS
		--------------------------------------------------------------------
		-- For Debugging
        --------------------------------------------------------------------
        LAX_CPU_I_ReadData        : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_I_WaitRequest     : Out Std_Logic;
        LAX_CPU_I_Address         : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_I_Read            : Out Std_Logic;
        LAX_CPU_I_ReadDataValid   : Out Std_Logic;
		LAX_CPU_D_Address         : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_ByteEnable      : Out Std_Logic_Vector(3  Downto 0);
        LAX_CPU_D_Read            : Out Std_Logic;
        LAX_CPU_D_Write           : Out Std_Logic;
        LAX_CPU_D_ReadData        : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_WriteData       : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_WaitRequest     : Out Std_Logic;
		LAX_CPU_D_ReadDataValid   : Out Std_Logic; -- IncludeIf_HasDataCache
				
		LAX_jtag_debug_module_debugaccess_to_roms : out std_logic; 					-- IncludeIf_OCDS
		LAX_jtag_debug_module_resetrequest        : out std_logic;					-- IncludeIf_OCDS
		LAX_jtag_debug_module_debugaccess         : out std_logic;					-- IncludeIf_OCDS
		LAX_jtag_debug_module_read                : out std_logic;					-- IncludeIf_OCDS
		LAX_jtag_debug_module_waitrequest         : out std_logic;					-- IncludeIf_OCDS
		LAX_jtag_debug_module_write               : out std_logic;					-- IncludeIf_OCDS

        Debug_MainMemory_I_Wait   : Out Std_Logic;
        Debug_MainMemory_D_Wait   : Out Std_Logic;
        Debug_Peripheral_I_Wait   : Out Std_Logic;
        Debug_Peripheral_D_Wait   : Out Std_Logic;
        Debug_InternalMem_I_Wait  : Out Std_Logic;
        Debug_InternalMem_D_Wait  : Out Std_Logic;
        Debug_DebugMem_I_Wait     : Out Std_Logic;
        Debug_DebugMem_D_Wait     : Out Std_Logic;
        Debug_TimeBase_I_Wait     : Out Std_Logic;
        Debug_TimeBase_D_Wait     : Out Std_Logic;
		
		Debug_State_IO            : Out Std_Logic_Vector(1 Downto 0);
        Debug_StateNext_IO        : Out Std_Logic_Vector(1 Downto 0);
        Debug_Access_IO_Data               : Out Std_Logic;
		Debug_Access_IO_Instr              : Out Std_Logic;
		Debug_Access_ME_Data               : Out Std_Logic;
		Debug_Access_ME_Instr              : Out Std_Logic;
		Debug_Access_ME                    : Out Std_Logic;
		Debug_Access_InternalMem_Data      : Out Std_Logic;
		Debug_Access_InternalMem_Instr     : Out Std_Logic;
		Debug_Access_DebugMem_Data         : Out Std_Logic;
		Debug_Access_DebugMem_Instr        : Out Std_Logic;
		Debug_Access_TimeBaseMem_Data      : Out Std_Logic;
		Debug_Access_TimeBaseMem_Instr     : Out Std_Logic;
		
		Debug_InstrAddressIsInternalMemory : Out Std_Logic;
		Debug_InstrAddressIsPeripheral     : Out Std_Logic;
		Debug_InstrAddressIsExternalMemory : Out Std_Logic;
		Debug_InstrAddressIsDebugMemory    : Out Std_Logic;
		Debug_InstrAddressIsTimeBaseMemory : Out Std_Logic;
		Debug_DataAddressIsInternalMemory  : Out Std_Logic;
		Debug_DataAddressIsPeripheral      : Out Std_Logic;
		Debug_DataAddressIsExternalMemory  : Out Std_Logic;
		Debug_DataAddressIsDebugMemory     : Out Std_Logic;
		Debug_DataAddressIsTimeBaseMemory  : Out Std_Logic
            --------------------------------------------------------------------
        );
    End Component;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Constant  BootAddressWidth : Integer := ReplaceWith_BootAddressWidth;
    Component Memory_ReplaceWith_Designator
        Port
        (
            ADDRA   : In  Std_Logic_Vector(BootAddressWidth-1 DownTo 0);
            ADDRB   : In  Std_Logic_Vector(BootAddressWidth-1 DownTo 0);
            ByteWEA : In  Std_Logic_Vector(3 DownTo 0);
            ByteWEB : In  Std_Logic_Vector(3 DownTo 0);
            CLKA    : In  Std_Logic;
            CLKB    : In  Std_Logic;
            DINA    : In  Std_Logic_Vector(31 DownTo 0);
            DINB    : In  Std_Logic_Vector(31 DownTo 0);
            DOUTA   : Out Std_Logic_Vector(31 DownTo 0);
            DOUTB   : Out Std_Logic_Vector(31 DownTo 0);
            ENA     : In  Std_Logic;
            ENB     : In  Std_Logic;
            WEA     : In  Std_Logic;
            WEB     : In  Std_Logic
        );
    End Component;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Component ReplaceWith_Designator_Nios2_TimeBase
        Port (
            -- Global signals
            CLK_I : In Std_Logic;
            RST_I : In Std_Logic;

            DAT_I : In  Std_Logic_Vector(31 Downto 0);
            DAT_O : Out Std_Logic_Vector(31 Downto 0);
            ADR_I : In  Std_Logic_Vector(1 Downto 0);
            WE_I  : In  Std_Logic;

            INT_O : Out Std_Logic
        );
    End Component;
    ----------------------------------------------------------------------------

    Signal Signal_GND4                             : Std_Logic_Vector(3 Downto 0);
    Signal Signal_GND                              : Std_Logic;
    Signal Signal_VCC                              : Std_Logic;
	Signal Signal_Rst 							   : Std_Logic;
    Signal Boot_Reset_Counter                      : Std_Logic_Vector(7 Downto 0);

    Signal PinSignal_SubPart_LowRAM_ADDRA          : STD_LOGIC_VECTOR(31 downto 0);
    Signal PinSignal_SubPart_LowRAM_ADDRB          : STD_LOGIC_VECTOR(31 downto 0);
    Signal PinSignal_SubPart_LowRAM_ByteWEA        : STD_LOGIC_VECTOR(3 downto 0);
    Signal PinSignal_SubPart_LowRAM_ByteWEB        : STD_LOGIC_VECTOR(3 downto 0);
    Signal PinSignal_SubPart_LowRAM_DINA           : STD_LOGIC_VECTOR(31 downto 0);
    Signal PinSignal_SubPart_LowRAM_DINB           : STD_LOGIC_VECTOR(31 downto 0);
    Signal PinSignal_SubPart_LowRAM_ENA            : STD_LOGIC;
    Signal PinSignal_SubPart_LowRAM_ENB            : STD_LOGIC;
    Signal PinSignal_SubPart_LowRAM_LOWRAM_DO      : STD_LOGIC_VECTOR(31 downto 0);
    Signal PinSignal_SubPart_LowRAM_WEA            : STD_LOGIC;
    Signal PinSignal_SubPart_LowRAM_WEB            : STD_LOGIC;
    Signal PinSignal_SubPart_LowRAM_DOUTA          : STD_LOGIC_VECTOR(31 downto 0);
    Signal PinSignal_SubPart_LowRAM_DOUTB          : STD_LOGIC_VECTOR(31 downto 0);

    Signal PinSignal_SubPart_Nios2_TimeBase_DAT_I  : Std_Logic_Vector(31 Downto 0);
    Signal PinSignal_SubPart_Nios2_TimeBase_DAT_O  : Std_Logic_Vector(31 Downto 0);
    Signal PinSignal_SubPart_Nios2_TimeBase_ADR_I  : Std_Logic_Vector(1 Downto 0);
    Signal PinSignal_SubPart_Nios2_TimeBase_WE_I   : Std_Logic;
    Signal PinSignal_SubPart_Nios2_TimeBase_INT_O  : Std_Logic;
    Signal PinSignal_SubPart_Nios2_TimeBase_RST_I  : Std_Logic;

    ----------------------------------------------------------------------------
    -- For Debugging -- 2014-09-03
    ----------------------------------------------------------------------------
	Signal Debug_ir_out_signal				: std_logic_vector(1 downto 0);
	Signal Debug_tdo_signal					: std_logic;
	Signal Debug_cdr_signal					: std_logic;
	Signal Debug_ir_in_signal				: std_logic_vector(1 downto 0);
	Signal Debug_rti_signal					: std_logic;
	Signal Debug_sdr_signal					: std_logic;
	Signal Debug_tck_signal					: std_logic;
	Signal Debug_tdi_signal					: std_logic;
	Signal Debug_udr_signal					: std_logic;
	Signal Debug_uir_signal					: std_logic;

    ----------------------------------------------------------------------------
Begin
-- 2014-09-18
    ----------------------------------------------------------------------------
	LAX_TAP_ir_out 	<= Debug_ir_out_signal;	-- IncludeIf_OCDS
	LAX_TAP_tdo		<= Debug_tdo_signal;	-- IncludeIf_OCDS
	LAX_TAP_cdr		<= Debug_cdr_signal;	-- IncludeIf_OCDS
	LAX_TAP_ir_in	<= Debug_ir_in_signal;	-- IncludeIf_OCDS
	LAX_TAP_rti		<= Debug_rti_signal;	-- IncludeIf_OCDS
	LAX_TAP_sdr		<= Debug_sdr_signal;	-- IncludeIf_OCDS
	LAX_TAP_tck		<= Debug_tck_signal;	-- IncludeIf_OCDS
	LAX_TAP_tdi		<= Debug_tdi_signal;	-- IncludeIf_OCDS
	LAX_TAP_udr		<= Debug_udr_signal;	-- IncludeIf_OCDS
	LAX_TAP_uir		<= Debug_uir_signal;	-- IncludeIf_OCDS

    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Signal_GND4                     <= (Others=>'0');
    Signal_GND                      <= '0';
    Signal_VCC                      <= '1';
    Signal_Rst                      <= Not Boot_Reset_Counter(7);

    PinSignal_SubPart_LowRAM_ByteWEA <= Signal_GND4;
    PinSignal_SubPart_LowRAM_WEA     <= Signal_GND;
    PinSignal_SubPart_LowRAM_ENA     <= Signal_VCC;
    PinSignal_SubPart_LowRAM_ENB     <= Signal_VCC;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    SubPart_OCDS : Nios2_OCDS							-- IncludeIf_OCDS
    Port Map (											-- IncludeIf_OCDS
        Altium_TCK     => TCK,							-- IncludeIf_OCDS
        Altium_TDI     => TDI,							-- IncludeIf_OCDS	
        Altium_TMS     => TMS,							-- IncludeIf_OCDS	
        Altium_TRST    => TRST,							-- IncludeIf_OCDS			
        Altium_TDO     => TDO,							-- IncludeIf_OCDS
        Reset_n        => Signal_VCC,					-- IncludeIf_OCDS
       -- Processor Debug Interface					
		vji_ir_out				=> Debug_ir_out_signal,	-- IncludeIf_OCDS
		vji_tdo					=> Debug_tdo_signal,	-- IncludeIf_OCDS	
		vji_cdr					=> Debug_cdr_signal,	-- IncludeIf_OCDS
		vji_ir_in				=> Debug_ir_in_signal,	-- IncludeIf_OCDS
		vji_rti					=> Debug_rti_signal,	-- IncludeIf_OCDS
		vji_sdr					=> Debug_sdr_signal,	-- IncludeIf_OCDS
		vji_tck					=> Debug_tck_signal,	-- IncludeIf_OCDS
		vji_tdi					=> Debug_tdi_signal,	-- IncludeIf_OCDS
		vji_udr					=> Debug_udr_signal,	-- IncludeIf_OCDS
		vji_uir					=> Debug_uir_signal,	-- IncludeIf_OCDS
		LAX_test_bus			=> LAX_OCDS_test_bus	-- IncludeIf_OCDS

    );-- IncludeIf_OCDS
    ----------------------------------------------------------------------------
	-- 2014-09-03
    ----------------------------------------------------------------------------
    SubPart_NIOS2 : ReplaceWith_Designator_Nios2_Template
    Port Map (
       CLK_I         		=> CLK_I,
        RST_I         		=> Signal_Rst,
        INT_I         		=> INT_I,

        IO_STB_O      		=> IO_STB_O,
        IO_CYC_O      		=> IO_CYC_O,
        IO_ACK_I      		=> IO_ACK_I,
        IO_ADR_O      		=> IO_ADR_O,
        IO_DAT_I      		=> IO_DAT_I,
        IO_DAT_O      		=> IO_DAT_O,
        IO_SEL_O      		=> IO_SEL_O,
        IO_WE_O       		=> IO_WE_O ,
        IO_CLK_O      		=> IO_CLK_O,
        IO_RST_O      		=> IO_RST_O,

        -- RAMA is for read and RAMB is for write
        InstrRomAddr  		=> PinSignal_SubPart_LowRAM_ADDRA,
        InstrRomIn    		=> PinSignal_SubPart_LowRAM_DOUTA,
        --DataRamOut  		=> PinSignal_SubPart_LowRAM_DINA,
        DataRamAddr   		=> PinSignal_SubPart_LowRAM_ADDRB,
        DataRamBE     		=> PinSignal_SubPart_LowRAM_ByteWEB,
        DataRamOut    		=> PinSignal_SubPart_LowRAM_DINB,
        DataRamIn     		=> PinSignal_SubPart_LowRAM_DOUTB,
        DataRamWE     		=> PinSignal_SubPart_LowRAM_WEB,
				
		ME_STB_O      		=> ME_STB_O,
        ME_CYC_O      		=> ME_CYC_O,
        ME_ACK_I      		=> ME_ACK_I,
        ME_ADR_O      		=> ME_ADR_O,
        ME_DAT_I      		=> ME_DAT_I,
        ME_DAT_O      		=> ME_DAT_O,
        ME_SEL_O      		=> ME_SEL_O,
        ME_WE_O       		=> ME_WE_O ,
        ME_CLK_O      		=> ME_CLK_O,
        ME_RST_O      		=> ME_RST_O,

        TimeBaseIn          => PinSignal_SubPart_Nios2_TimeBase_DAT_O,
        TimeBaseOut         => PinSignal_SubPart_Nios2_TimeBase_DAT_I,
        TimeBaseAddr        => PinSignal_SubPart_Nios2_TimeBase_ADR_I,
        TimeBaseWE          => PinSignal_SubPart_Nios2_TimeBase_WE_I,
        TimeBaseInterrupt   => PinSignal_SubPart_Nios2_TimeBase_INT_O,
        TimeBaseReset       => PinSignal_SubPart_Nios2_TimeBase_RST_I,
		
		JTAG_ir_out			=> Debug_ir_out_signal,	-- IncludeIf_OCDS
		JTAG_tdo			=> Debug_tdo_signal,	-- IncludeIf_OCDS
		JTAG_cdr			=> Debug_cdr_signal,	-- IncludeIf_OCDS
		JTAG_ir_in			=> Debug_ir_in_signal,	-- IncludeIf_OCDS
		JTAG_rti			=> Debug_rti_signal,	-- IncludeIf_OCDS
		JTAG_sdr			=> Debug_sdr_signal,	-- IncludeIf_OCDS
		JTAG_tck			=> Debug_tck_signal,	-- IncludeIf_OCDS
		JTAG_tdi			=> Debug_tdi_signal,	-- IncludeIf_OCDS
		JTAG_udr			=> Debug_udr_signal,	-- IncludeIf_OCDS
		JTAG_uir			=> Debug_uir_signal,	-- IncludeIf_OCDS
		-- For Debugging
		LAX_CPU_I_ReadData        => LAX_CPU_I_ReadData,
        LAX_CPU_I_WaitRequest     => LAX_CPU_I_WaitRequest,
        LAX_CPU_I_Address         => LAX_CPU_I_Address,
        LAX_CPU_I_Read            => LAX_CPU_I_Read,
        LAX_CPU_I_ReadDataValid   => LAX_CPU_I_ReadDataValid,	 
        LAX_CPU_D_Address         => LAX_CPU_D_Address,
        LAX_CPU_D_ByteEnable      => LAX_CPU_D_ByteEnable,
        LAX_CPU_D_Read            => LAX_CPU_D_Read,
        LAX_CPU_D_Write           => LAX_CPU_D_Write,
        LAX_CPU_D_ReadData        => LAX_CPU_D_ReadData,
        LAX_CPU_D_WriteData       => LAX_CPU_D_WriteData,
        LAX_CPU_D_WaitRequest     => LAX_CPU_D_WaitRequest,
		LAX_CPU_D_ReadDataValid   => LAX_CPU_D_ReadDataValid, -- IncludeIf_HasDataCache
		
		LAX_jtag_debug_module_debugaccess_to_roms	=> LAX_jtag_module_debugaccess_to_roms,	-- IncludeIf_OCDS
        LAX_jtag_debug_module_resetrequest			=> LAX_jtag_module_resetrequest,		-- IncludeIf_OCDS
		LAX_jtag_debug_module_debugaccess			=> LAX_jtag_module_debugaccess,			-- IncludeIf_OCDS
		LAX_jtag_debug_module_read					=> LAX_jtag_module_read,				-- IncludeIf_OCDS
		LAX_jtag_debug_module_waitrequest			=> LAX_jtag_module_waitrequest,			-- IncludeIf_OCDS
		LAX_jtag_debug_module_write					=> LAX_jtag_module_write,				-- IncludeIf_OCDS
		
		
        Debug_MainMemory_I_Wait    => LAX_MainMemory_I_Wait,
        Debug_MainMemory_D_Wait    => LAX_MainMemory_D_Wait,
        Debug_Peripheral_I_Wait    => LAX_Peripheral_I_Wait,
        Debug_Peripheral_D_Wait    => LAX_Peripheral_D_Wait,
        Debug_InternalMem_I_Wait   => LAX_InternalMem_I_Wait,
        Debug_InternalMem_D_Wait   => LAX_InternalMem_D_Wait,
        Debug_DebugMem_I_Wait      => LAX_DebugMem_I_Wait,
        Debug_DebugMem_D_Wait      => LAX_DebugMem_D_Wait,
		Debug_TimeBase_I_Wait      => LAX_TimeBase_I_Wait,
        Debug_TimeBase_D_Wait      => LAX_TimeBase_D_Wait,
        
		Debug_State_IO             => LAX_Debug_State_IO,
        Debug_StateNext_IO         => LAX_Debug_StateNext_IO,
        Debug_Access_IO_Data 	   => LAX_Debug_Access_IO_Data,
		Debug_Access_IO_Instr	   => LAX_Debug_Access_IO_Instr,
		Debug_Access_ME_Data			=> LAX_Debug_Access_ME_Data,
		Debug_Access_ME_Instr			=> LAX_Debug_Access_ME_Instr,
		Debug_Access_ME      			=> LAX_Debug_Access_ME,
		Debug_Access_InternalMem_Data	=> LAX_Debug_Access_InternalMem_Data,
		Debug_Access_InternalMem_Instr	=> LAX_Debug_Access_InternalMem_Instr,
		Debug_Access_DebugMem_Data    	=> LAX_Debug_Access_DebugMem_Data,
		Debug_Access_DebugMem_Instr   	=> LAX_Debug_Access_DebugMem_Instr,
		Debug_Access_TimeBaseMem_Data 	=> LAX_Debug_Access_TimeBaseMem_Data,
		Debug_Access_TimeBaseMem_Instr	=> LAX_Debug_Access_TimeBaseMem_Instr,
		
		Debug_InstrAddressIsInternalMemory  => LAX_I_AddressIsInternalMemory,
		Debug_InstrAddressIsPeripheral      => LAX_I_AddressIsPeripheral,
		Debug_InstrAddressIsExternalMemory  => LAX_I_AddressIsExternalMemory,
		Debug_InstrAddressIsDebugMemory     => LAX_I_AddressIsDebugMemory,
		Debug_InstrAddressIsTimeBaseMemory  => LAX_I_AddressIsTimeBaseMemory,
		Debug_DataAddressIsInternalMemory   => LAX_D_AddressIsInternalMemory,
		Debug_DataAddressIsPeripheral       => LAX_D_AddressIsPeripheral,
		Debug_DataAddressIsExternalMemory   => LAX_D_AddressIsExternalMemory,
		Debug_DataAddressIsDebugMemory      => LAX_D_AddressIsDebugMemory,
		Debug_DataAddressIsTimeBaseMemory   => LAX_D_AddressIsTimeBaseMemory
   );
   ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    SubPart_BootMemory : Memory_ReplaceWith_Designator
    Port Map
    (
        ADDRA      => PinSignal_SubPart_LowRAM_ADDRA(BootAddressWidth+1 DownTo 2),
        ADDRB      => PinSignal_SubPart_LowRAM_ADDRB(BootAddressWidth+1 DownTo 2),
        ByteWEA    => Signal_GND4,
        ByteWEB    => PinSignal_SubPart_LowRAM_ByteWEB,
        DINA       => (Others => '0'),
        DINB       => PinSignal_SubPart_LowRAM_DINB,
        DOUTA      => PinSignal_SubPart_LowRAM_DOUTA,
        DOUTB      => PinSignal_SubPart_LowRAM_DOUTB,
        ENA        => Signal_VCC,
        ENB        => Signal_VCC,
        WEA        => Signal_GND,
        WEB        => PinSignal_SubPart_LowRAM_WEB,
        CLKA       => CLK_I,
        CLKB       => CLK_I
    );
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Boot_Reset:
    Process(CLK_I,
            RST_I)
    Begin
        If Rising_Edge(CLK_I) Then
            If RST_I = '1' Then
                Boot_Reset_Counter <= (Others=>'0');
            ElsIf Boot_Reset_Counter(7) = '0' Then
                Boot_Reset_Counter <= Boot_Reset_Counter + 1 ;
            Else
                Boot_Reset_Counter <= Boot_Reset_Counter;
            End If ;
        End if ;
    End Process ;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    SubPart_Nios2_TimeBase : ReplaceWith_Designator_Nios2_TimeBase Port Map (
        CLK_I => CLK_I,
        RST_I => PinSignal_SubPart_Nios2_TimeBase_RST_I,

        DAT_I => PinSignal_SubPart_Nios2_TimeBase_DAT_I,
        DAT_O => PinSignal_SubPart_Nios2_TimeBase_DAT_O,
        ADR_I => PinSignal_SubPart_Nios2_TimeBase_ADR_I,
        WE_I  => PinSignal_SubPart_Nios2_TimeBase_WE_I,

        INT_O => PinSignal_SubPart_Nios2_TimeBase_INT_O
    );
    ----------------------------------------------------------------------------

------------------------------------------------------------------------------------
End RTL;
------------------------------------------------------------------------------------

--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
Use IEEE.Std_Logic_Unsigned.All;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
Entity ReplaceWith_Designator_Nios2_TimeBase Is
    Port
    (
        -- Global signals
        CLK_I : In Std_Logic;
        RST_I : In Std_Logic;

        DAT_I : In  Std_Logic_Vector(31 Downto 0);
        DAT_O : Out Std_Logic_Vector(31 Downto 0);
        ADR_I : In  Std_Logic_Vector(1 Downto 0);
        WE_I  : In  Std_Logic;

        INT_O : Out Std_Logic
    );
End ReplaceWith_Designator_Nios2_TimeBase;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
Architecture RTL Of ReplaceWith_Designator_Nios2_TimeBase Is
    Signal    TimeBaseRegister_Count    : Std_Logic_Vector(63 DownTo 0);   -- Generic clock counter
    Signal    TimeBaseRegister_Lo       : Std_Logic_Vector(31 DownTo 0);
    Signal    TimeBaseRegister_Hi       : Std_Logic_Vector(31 DownTo 0);
    Signal    TimeBaseRegister_Transfer : Std_Logic_Vector(31 DownTo 0);

    Signal    TimerControlRegister      : Std_Logic_Vector(2 Downto 0);
    Signal    TimerMaxValueRegister     : Std_Logic_Vector(31 Downto 0);
    Signal    Timer                     : Std_Logic_Vector(31 Downto 0);

    Constant  cTimeBaseRegister_LO      : Std_Logic_Vector(1 Downto 0) := "00";
    Constant  cTimeBaseRegister_HI      : Std_Logic_Vector(1 Downto 0) := "01";
    Constant  cTimerMaxValueRegister    : Std_Logic_Vector(1 Downto 0) := "10";
    Constant  cTimerControlRegister     : Std_Logic_Vector(1 Downto 0) := "11";

    Signal    Interrupt                 : Std_Logic;

    Constant  cTimerEnable              : Integer := 0;
    Constant  cInterruptPending         : Integer := 1;
    Constant  cClearInterrupt           : Integer := 2;
    Constant  cResetTimer               : Integer := 3;
Begin
    ----------------------------------------------------------------------------
    -- 64-bit counter - always counts on every clock.
    TickCounterIncrement:
    Process(CLK_I)
    Begin
        If Rising_Edge(CLK_I) Then
            If RST_I = '1' Then
                TimeBaseRegister_Count <= (Others => '0');
            ElsIf (WE_I = '1') And (ADR_I = cTimerControlRegister) And (DAT_I(cResetTimer) = '1') Then
                TimeBaseRegister_Count <= (Others => '0');
            Else
                TimeBaseRegister_Count <= TimeBaseRegister_Count + 1;
            End If;
       End If;
    End Process;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    TickCounterCapture:
    Process (CLK_I)
    Begin
        If Rising_Edge(CLK_I) Then
           If RST_I = '1' Then
              TimeBaseRegister_Lo       <= (Others => '0');
              TimeBaseRegister_Hi       <= (Others => '0');
           Else
               If WE_I='0' Then
                  Case ADR_I Is
                    When cTimeBaseRegister_LO =>
                     -- Capture Timer Register
                        TimeBaseRegister_LO <= TimeBaseRegister_Count(31 DownTo 0);
                        TimeBaseRegister_Hi <= TimeBaseRegister_Count(63 DownTo 32);
                    When Others =>
                        TimeBaseRegister_Lo <= TimeBaseRegister_Lo;
                        TimeBaseRegister_Hi <= TimeBaseRegister_Hi;
                  End Case;

               End If;
           End If;
        End If;
    End Process;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    TimerIncrement:
    Process(CLK_I)
    Begin
        If Rising_Edge(CLK_I) Then
            If RST_I='1' Then
                Timer <= (Others=>'1');
            Else
                If TimerControlRegister(cTimerEnable) = '1' Then
                    If Timer = TimerMaxValueRegister Then
                        Timer <= (Others=>'0');
                    Else
                        Timer <= Timer + 1;
                    End If;
                End If;
            End If;
        End If;
    End Process;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    TimerInterrupt:
    Process(CLK_I)
    Begin
        If Rising_Edge(CLK_I) Then
            If RST_I='1' Then
                Interrupt <= '0';
            Else
                If TimerControlRegister(cTimerEnable) = '1' Then
                    If Timer = TimerMaxValueRegister Then
                        Interrupt <= '1';
                    ElsIf (WE_I = '1') And (ADR_I = cTimerControlRegister) And (DAT_I(cClearInterrupt) = '1') Then
                        Interrupt <= '0';
                    End If;
                Else
                    Interrupt <= '0';
                End If;
            End If;
        End If;
    End Process;
    ----------------------------------------------------------------------------

    INT_O <= Interrupt;

    ----------------------------------------------------------------------------
    RegisterWrite:
    Process(CLK_I)
    Begin
        If Rising_Edge(CLK_I) Then
            If RST_I='1' Then
                TimerMaxValueRegister <= (Others=>'0');
                TimerControlRegister  <= (Others=>'0');
            Else
                If WE_I='1' Then
                    Case ADR_I Is
                        When cTimerMaxValueRegister =>
                            TimerMaxValueRegister <= DAT_I;
                        When cTimerControlRegister =>
                            TimerControlRegister <= DAT_I(2 Downto 0);
                        When Others =>
                            NULL;
                    End Case;
                End If;
            End If;
        End If;
    End Process;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    RegisterRead :
    Process(TimeBaseRegister_Lo,
            TimeBaseRegister_Count,
            TimerMaxValueRegister,
            ADR_I,
            Interrupt,
            TimerControlRegister)
    Begin
        Case ADR_I Is
            When cTimeBaseRegister_LO =>
                 DAT_O <= TimeBaseRegister_Lo;
            When cTimeBaseRegister_HI =>
                 DAT_O <= TimeBaseRegister_Count(63 Downto 32);
            When cTimerMaxValueRegister =>
                 DAT_O <= TimerMaxValueRegister;
            When cTimerControlRegister =>
                 DAT_O <= "000000000000000000000000000000" & Interrupt & TimerControlRegister(cTimerEnable);
            When Others =>
                 DAT_O <= (Others=>'0');
        End Case;
    End Process;
    ----------------------------------------------------------------------------

End RTL;
--------------------------------------------------------------------------------
-- Round-Robin Arbitrator -- 2014-09-03
------------------------------------------------------------------------------------
Library IEEE;
	Use IEEE.Std_Logic_1164.All;
	Use IEEE.Std_Logic_Unsigned.All;
	use ieee.numeric_std.all;
	
Entity ReplaceWith_Designator_nios_master_rr_arbiter Is
    Port(
			CLK_I	: In  Std_Logic;
			RST_I	: In  Std_Logic;
			REQ_I	: In  Std_Logic_Vector(1 downto 0);
			GRANT_O	: OUT Std_Logic_Vector(1 downto 0)
	);		
End ReplaceWith_Designator_nios_master_rr_arbiter;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
Architecture RTL Of ReplaceWith_Designator_nios_master_rr_arbiter Is
	type round_robin_state_type is(			-- State declaration
							idle_st,
							s0,
							s1
						); 
	Signal grant_vec	: Std_Logic_Vector(1 downto 0) := (others => '0');
	Signal request_vec	: Std_Logic_Vector(1 downto 0) := (others => '0');
	signal rr_state		: round_robin_state_type := idle_st;
	signal rr_nxt_state	: round_robin_state_type := idle_st;
Begin
	request_vec	<= REQ_I;
	
	priority : process( CLK_I, RST_I, rr_state, request_vec	)
		-- Previous REQUEST vector
		variable	prev_request_vec : std_logic_vector(1 downto 0) := B"00";
	
	begin  -- process priority
		case rr_state is
			when idle_st =>				-- IDLE State
				if(request_vec(0)='1')then
					grant_vec		<= "01";
					rr_nxt_state	<= s0;
				elsif request_vec(1)='1' then
					grant_vec		<="10";
					rr_nxt_state	<= s1;
				else
					grant_vec		<="00";
					rr_nxt_state	<= idle_st;
				end if;
			when s0 =>
				if request_vec(0)='1' and prev_request_vec(0) = '1' then
					grant_vec		<= "01";
					rr_nxt_state	<= s0;
				elsif(request_vec(1)='1')then
					grant_vec		<= "10";
					rr_nxt_state	<= s1;
				else
					grant_vec		<= "00";
					rr_nxt_state	<= idle_st;
				end if;
			when s1 =>
				if (request_vec(1)='1' and prev_request_vec(1) = '1') then
					grant_vec		<="10";
					rr_nxt_state 	<= s1;
				Elsif (request_vec(0)='1') then
					grant_vec		<="01";
					rr_nxt_state	<= s0;
				else
					grant_vec		<="00";
					rr_nxt_state	<= idle_st;
				end if;
			when others => 	
					grant_vec		<="00";
					rr_nxt_state 	<= idle_st;
					
		end case;
		prev_request_vec := request_vec;
	end process priority;
	GRANT_O	<= grant_vec;
	-- 
	priority_regs : process(CLK_I, RST_I) is
	begin
		if (RST_I = '1') then
			rr_state 	<= idle_st;
		elsif(CLK_I'event and CLK_I = '1') then
			rr_state <= rr_nxt_state;
		end if;	
	end process priority_regs;
end RTL;	
--------------------------------------------------------------------------------

------------------------------------------------------------------------------------
Library IEEE;
	Use IEEE.Std_Logic_1164.All;
	Use IEEE.Std_Logic_Unsigned.All;
	use IEEE.numeric_std.all;			-- 2014-09-03
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
Entity ReplaceWith_Designator_Nios2_Template Is			-- 2014-09-03
    Port (
		--------------------------------------------------------------------
        CLK_I                    : In  Std_Logic;
        RST_I                    : In  Std_Logic;
        INT_I                    : In  Std_Logic_Vector(31 Downto 0);
        --------------------------------------------------------------------
		-- Peripheral Interface
        --------------------------------------------------------------------
        IO_STB_O                 : Out Std_Logic;
        IO_CYC_O                 : Out Std_Logic;
        IO_ACK_I                 : In  Std_Logic;
        IO_ADR_O                 : Out Std_Logic_Vector(23 Downto 0);
        IO_DAT_I                 : In  Std_Logic_Vector(31 Downto 0);
        IO_DAT_O                 : Out Std_Logic_Vector(31 Downto 0);
        IO_SEL_O                 : Out Std_Logic_Vector(3  Downto 0);
        IO_WE_O                  : Out Std_Logic;
        IO_CLK_O                 : Out Std_Logic;
        IO_RST_O                 : Out Std_Logic;
        --------------------------------------------------------------------
		-- Internal Memory Interface
        --------------------------------------------------------------------
        DataRamIn                : In  Std_Logic_Vector(31 Downto 0);
        DataRamOut               : Out Std_Logic_Vector(31 Downto 0);
        DataRamBE                : Out Std_Logic_Vector(3  Downto 0);
        DataRamWE                : Out Std_Logic;
        DataRamAddr              : Out Std_Logic_Vector(31 Downto 0);
        InstrRomIn               : In  Std_Logic_Vector(31 Downto 0);
        InstrRomAddr             : Out Std_Logic_Vector(31 Downto 0);
        --------------------------------------------------------------------
		-- External Memory Interface
        --------------------------------------------------------------------
        ME_STB_O                 : Out Std_Logic;
        ME_CYC_O                 : Out Std_Logic;
        ME_ACK_I                 : In  Std_Logic;
        ME_ADR_O                 : Out Std_Logic_Vector(31 Downto 0);
        ME_DAT_I                 : In  Std_Logic_Vector(31 Downto 0);
        ME_DAT_O                 : Out Std_Logic_Vector(31 Downto 0);
        ME_SEL_O                 : Out Std_Logic_Vector(3  Downto 0);
        ME_WE_O                  : Out Std_Logic;
        ME_CLK_O                 : Out Std_Logic;
        ME_RST_O                 : Out Std_Logic;
        --------------------------------------------------------------------
		-- TimeBase Memory Interface
        --------------------------------------------------------------------
        TimeBaseIn               	: In  Std_Logic_Vector(31 Downto 0);
        TimeBaseOut              	: Out Std_Logic_Vector(31 Downto 0);
        TimeBaseAddr             	: Out Std_Logic_Vector(1 Downto 0);
        TimeBaseWE               	: Out Std_Logic;
        TimeBaseInterrupt        	: In  Std_Logic;
        TimeBaseReset            	: Out Std_Logic;
		--------------------------------------------------------------------
		-- Virtual JTAG Module Interface
        --------------------------------------------------------------------
		JTAG_ir_out					: out std_logic_vector(1 downto 0);		-- IncludeIf_OCDS
		JTAG_tdo					: out std_logic;						-- IncludeIf_OCDS		
		JTAG_cdr					: in  std_logic;						-- IncludeIf_OCDS
		JTAG_ir_in					: in  std_logic_vector(1 downto 0);		-- IncludeIf_OCDS	
		JTAG_rti					: in  std_logic;						-- IncludeIf_OCDS
		JTAG_sdr					: in  std_logic;						-- IncludeIf_OCDS
		JTAG_tck					: in  std_logic;						-- IncludeIf_OCDS
		JTAG_tdi					: in  std_logic;						-- IncludeIf_OCDS
		JTAG_udr					: in  std_logic;						-- IncludeIf_OCDS
		JTAG_uir					: in  std_logic;						-- IncludeIf_OCDS
		--------------------------------------------------------------------
		-- For Debugging
        --------------------------------------------------------------------
        LAX_CPU_I_ReadData        : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_I_WaitRequest     : Out Std_Logic;
        LAX_CPU_I_Address         : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_I_Read            : Out Std_Logic;
        LAX_CPU_I_ReadDataValid   : Out Std_Logic;
		LAX_CPU_D_Address         : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_ByteEnable      : Out Std_Logic_Vector(3  Downto 0);
        LAX_CPU_D_Read            : Out Std_Logic;
        LAX_CPU_D_Write           : Out Std_Logic;
        LAX_CPU_D_ReadData        : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_WriteData       : Out Std_Logic_Vector(31 Downto 0);
        LAX_CPU_D_WaitRequest     : Out Std_Logic;
		LAX_CPU_D_ReadDataValid   : Out Std_Logic; -- IncludeIf_HasDataCache
				
		LAX_jtag_debug_module_debugaccess_to_roms : out std_logic; 	-- IncludeIf_OCDS
		LAX_jtag_debug_module_resetrequest        : out std_logic;	-- IncludeIf_OCDS
		LAX_jtag_debug_module_debugaccess         : out std_logic;	-- IncludeIf_OCDS
		LAX_jtag_debug_module_read                : out std_logic;	-- IncludeIf_OCDS
		LAX_jtag_debug_module_waitrequest         : out std_logic;	-- IncludeIf_OCDS
		LAX_jtag_debug_module_write               : out std_logic;	-- IncludeIf_OCDS

        Debug_MainMemory_I_Wait   : Out Std_Logic;
        Debug_MainMemory_D_Wait   : Out Std_Logic;
        Debug_Peripheral_I_Wait   : Out Std_Logic;
        Debug_Peripheral_D_Wait   : Out Std_Logic;
        Debug_InternalMem_I_Wait  : Out Std_Logic;
        Debug_InternalMem_D_Wait  : Out Std_Logic;
        Debug_DebugMem_I_Wait     : Out Std_Logic;
        Debug_DebugMem_D_Wait     : Out Std_Logic;
		Debug_TimeBase_I_Wait     : Out Std_Logic;
        Debug_TimeBase_D_Wait     : Out Std_Logic;
		
        Debug_State_IO            : Out Std_Logic_Vector(1 Downto 0);
        Debug_StateNext_IO        : Out Std_Logic_Vector(1 Downto 0);
        Debug_Access_IO_Data               : Out Std_Logic;
		Debug_Access_IO_Instr              : Out Std_Logic;
		Debug_Access_ME_Data               : Out Std_Logic;
		Debug_Access_ME_Instr              : Out Std_Logic;
		Debug_Access_ME                    : Out Std_Logic;
		Debug_Access_InternalMem_Data      : Out Std_Logic;
		Debug_Access_InternalMem_Instr     : Out Std_Logic;
		Debug_Access_DebugMem_Data         : Out Std_Logic;
		Debug_Access_DebugMem_Instr        : Out Std_Logic;
		Debug_Access_TimeBaseMem_Data      : Out Std_Logic;
		Debug_Access_TimeBaseMem_Instr     : Out Std_Logic;
		
		Debug_InstrAddressIsInternalMemory : Out Std_Logic;
		Debug_InstrAddressIsPeripheral     : Out Std_Logic;
		Debug_InstrAddressIsExternalMemory : Out Std_Logic;
		Debug_InstrAddressIsDebugMemory    : Out Std_Logic;
		Debug_InstrAddressIsTimeBaseMemory : Out Std_Logic;
		Debug_DataAddressIsInternalMemory  : Out Std_Logic;
		Debug_DataAddressIsPeripheral      : Out Std_Logic;
		Debug_DataAddressIsExternalMemory  : Out Std_Logic;
		Debug_DataAddressIsDebugMemory     : Out Std_Logic;
		Debug_DataAddressIsTimeBaseMemory  : Out Std_Logic
        --------------------------------------------------------------------
   );
End;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
Architecture RTL Of ReplaceWith_Designator_Nios2_Template Is
------------------------------------------------------------------------------------
    --------------------------------------------------------------------
    Component ReplaceWith_MCU_CPU Is
        Port (
			Clk             : In  Std_Logic;
            Reset_n         : In  Std_Logic;

            I_ReadData      : In  Std_Logic_Vector(31 Downto 0);
            I_WaitRequest   : In  Std_Logic;
            I_Address       : Out Std_Logic_Vector(31 Downto 0);
            I_Read          : Out Std_Logic;
            I_ReadDataValid : In  Std_Logic;                 -- IncludeIf_HasInstructionCache

            D_Address       : Out Std_Logic_Vector(31 Downto 0);
            D_ByteEnable    : Out Std_Logic_Vector(3  Downto 0);
            D_Read          : Out Std_Logic;
            D_Write         : Out Std_Logic;
            D_ReadData      : In  Std_Logic_Vector(31 Downto 0);
            D_WriteData     : Out Std_Logic_Vector(31 Downto 0);
            D_WaitRequest   : In  Std_Logic;
			D_ReadDataValid : In  Std_Logic;       -- IncludeIf_HasDataCache
			--------------------------------------------------------------------
           
			-- Processor Debug Interface
			vji_ir_out                            : out std_logic_vector(1 downto 0);	-- IncludeIf_OCDS
			vji_tdo                               : out std_logic;						-- IncludeIf_OCDS
			vji_cdr                               : in  std_logic;						-- IncludeIf_OCDS
			vji_ir_in                             : in  std_logic_vector(1 downto 0);	-- IncludeIf_OCDS
			vji_rti                               : in  std_logic;						-- IncludeIf_OCDS
			vji_sdr                               : in  std_logic;						-- IncludeIf_OCDS
			vji_tck                               : in  std_logic;						-- IncludeIf_OCDS
			vji_tdi                               : in  std_logic;						-- IncludeIf_OCDS
			vji_udr                               : in  std_logic;						-- IncludeIf_OCDS
			vji_uir                               : in  std_logic;						-- IncludeIf_OCDS
			
			--no_ci_readra                          : out std_logic;
			
			jtag_debug_module_debugaccess_to_roms : out std_logic;						-- IncludeIf_OCDS 
            jtag_debug_module_resetrequest        : out std_logic;						-- IncludeIf_OCDS
			jtag_debug_module_address             : in  std_logic_vector(8 downto 0);	-- IncludeIf_OCDS
			jtag_debug_module_byteenable          : in  std_logic_vector(3 downto 0);	-- IncludeIf_OCDS
			jtag_debug_module_debugaccess         : in  std_logic;						-- IncludeIf_OCDS
			jtag_debug_module_read                : in  std_logic;						-- IncludeIf_OCDS
			jtag_debug_module_readdata            : out std_logic_vector(31 downto 0);	-- IncludeIf_OCDS
			jtag_debug_module_waitrequest         : out std_logic;						-- IncludeIf_OCDS
			jtag_debug_module_write               : in  std_logic;						-- IncludeIf_OCDS
			jtag_debug_module_writedata           : in  std_logic_vector(31 downto 0);	-- IncludeIf_OCDS
			
			D_Irq           : In  Std_Logic_Vector(31 Downto 0)
        );
    End Component;
    --------------------------------------------------------------------
	Component ReplaceWith_Designator_nios_master_rr_arbiter Is
    Port(
			CLK_I	: In  Std_Logic;
			RST_I	: In  Std_Logic;
			REQ_I	: In  Std_Logic_Vector(1 downto 0);
			GRANT_O	: OUT Std_Logic_Vector(1 downto 0)
	);		
	End Component;
------------------------------------------------------------------------------------

    ----------------------------------------------------------------------------
	Constant instr_mst				: integer range 0 to 1 := 0;	-- Nios Instruction Master Index
	Constant data_mst				: integer range 0 to 1 := 1;	-- Nios Data Master Index
    ----------------------------------------------------------------------------
	-- Addresses Constants
    ----------------------------------------------------------------------------
    Constant INT_MEM_UNIT_BASE_ADDR   	: unsigned(31 Downto 0) := x"00000000";
	Constant INT_MEM_UNIT_ADDR_LEN   	: unsigned(31 Downto 0) := x"00FFF000";
	
	Constant EXT_MEM_UNIT_BASE_ADDR   	: unsigned(31 Downto 0) := x"01000000";
	Constant EXT_MEM_UNIT_ADDR_LEN   	: unsigned(31 Downto 0) := x"7F000000";
	
	Constant DEBUG_UNIT_BASE_ADDR   	: unsigned(31 Downto 0) := x"00FFF000";
	Constant DEBUG_UNIT_ADDR_LEN   		: unsigned(31 Downto 0) := x"00000800";
	
	Constant PERIPHERAL_UNIT_BASE_ADDR  : unsigned(32 Downto 0) := B"0" & x"FF000000";
	Constant PERIPHERAL_UNIT_ADDR_LEN   : unsigned(32 Downto 0) := B"0" & x"00FFFF00";
	
    Constant TIMEBASE_UNIT_BASE_ADDR	: unsigned(32 Downto 0) := B"0" & x"FFFFFFE0";
	Constant TIMEBASE_UNIT_ADDR_LEN		: unsigned(32 Downto 0) := B"0" & x"00000020";
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    Signal Combined_Rst                 : Std_Logic;
    Signal Signal_Rst                   : Std_Logic;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Signal Instr_ReadData               : Std_Logic_Vector(31 Downto 0);
    Signal Instr_WaitRequest            : Std_Logic := '1';
    Signal Instr_Address                : Std_Logic_Vector(31 Downto 0);
    Signal Instr_Read                   : Std_Logic;
    Signal Instr_ReadDataValid          : Std_Logic;
    Signal Data_Address                 : Std_Logic_Vector(31 Downto 0);
    Signal Data_ByteEnable              : Std_Logic_Vector(3  Downto 0);
    Signal Data_Read                    : Std_Logic;
    Signal Data_Write                   : Std_Logic;
    Signal Data_ReadData                : Std_Logic_Vector(31 Downto 0);
    Signal Data_WriteData               : Std_Logic_Vector(31 Downto 0);
    Signal Data_WaitRequest				: Std_Logic := '1';
    Signal Data_ReadDataValid			: Std_Logic := '0';	
	----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Signal InstrAddressIsInternalMemory : Std_Logic := '0';
    Signal InstrAddressIsPeripheral     : Std_Logic := '0';
    Signal InstrAddressIsExternalMemory : Std_Logic := '0';
    Signal InstrAddressIsDebugMemory    : Std_Logic := '0';
    Signal InstrAddressIsTimeBaseMemory : Std_Logic := '0';
    Signal DataAddressIsInternalMemory  : Std_Logic := '0';
    Signal DataAddressIsPeripheral      : Std_Logic := '0';
    Signal DataAddressIsExternalMemory  : Std_Logic := '0';
    Signal DataAddressIsDebugMemory     : Std_Logic := '0';
    Signal DataAddressIsTimeBaseMemory  : Std_Logic := '0';
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Signal Access_ME_Data               : Std_Logic := '0';
    Signal Access_ME_Instr              : Std_Logic := '0';
    Signal Access_ME                    : Std_Logic := '0';
    Signal Access_IO_Data               : Std_Logic := '0';
    Signal Access_IO_Instr              : Std_Logic := '0';
    Signal Access_InternalMem_Data      : Std_Logic := '0';
    Signal Access_InternalMem_Instr     : Std_Logic := '0';
    Signal Access_DebugMem_Data         : Std_Logic := '0';
    Signal Access_DebugMem_Instr        : Std_Logic := '0';
    Signal Access_TimeBaseMem_Data      : Std_Logic := '0';
    Signal Access_TimeBaseMem_Instr     : Std_Logic := '0';
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Signal TimeOut_IO_Counter           : Std_Logic_Vector(12 Downto 0);
    Signal TimeOut_IO                   : Std_Logic;
    Signal TimeOut_ME_Counter           : Std_Logic_Vector(12 Downto 0);
    Signal TimeOut_ME                   : Std_Logic;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Signal MainMemory_I_Wait            : Std_Logic := '0';
    Signal MainMemory_D_Wait            : Std_Logic := '0';
    Signal Peripheral_I_Wait            : Std_Logic := '0';
    Signal Peripheral_D_Wait            : Std_Logic := '0';
    Signal InternalMem_I_Wait           : Std_Logic := '0';
    Signal InternalMem_D_Wait           : Std_Logic := '0';
    Signal DebugMem_I_Wait              : Std_Logic := '0';
    Signal DebugMem_D_Wait              : Std_Logic := '0';
    Signal TimeBaseMem_I_Wait           : Std_Logic := '0';
    Signal TimeBaseMem_D_Wait           : Std_Logic := '0';
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Signal MainMemory_Running           : Std_Logic;
    Signal Peripheral_Running           : Std_Logic;
    Signal Peripheral_Acked             : Std_Logic;
    Signal Peripheral_Data_In_Latch     : Std_Logic_Vector(31 downto 0);
    ----------------------------------------------------------------------------
 
    ----------------------------------------------------------------------------
    Signal Jtag_Debug_Module_Address             : Std_Logic_Vector(8 Downto 0);
    Signal Jtag_Debug_Module_ByteEnable          : Std_Logic_Vector(3 Downto 0);
    Signal Jtag_Debug_Module_DebugAccess         : Std_Logic;
    Signal Jtag_Debug_Module_Write               : Std_Logic;
    Signal Jtag_Debug_Module_WriteData           : Std_Logic_Vector(31 Downto 0);
    Signal Jtag_Debug_Module_DebugAccess_To_Roms : Std_Logic;
    Signal Jtag_Debug_Module_ReadData            : Std_Logic_Vector(31 Downto 0);
    Signal Jtag_Debug_Module_ResetRequest        : Std_Logic := '0';
	Signal Jtag_Debug_Module_Read				 : Std_Logic;
	Signal jtag_Debug_Module_Waitrequest	 	 : Std_Logic;
    ----------------------------------------------------------------------------
  
    ----------------------------------------------------------------------------
    Signal EndianFixed_Data_WriteData   : Std_Logic_Vector(31 Downto 0);
    Signal EndianFixed_Data_ByteEnable  : Std_Logic_Vector(3 Downto 0);
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    Signal Interrupt_Cpu                : Std_Logic_Vector(31 Downto 0);
	Signal test_bus						: Std_Logic_Vector(15 downto 0);
	
	-- Avalon Slave FSM Type
	type avs_state_type is(	
						idle_st,
						instr_access_st,
						data_access_st
	);
	----------------------------------------------------------------------------
	-- Signals for JTAG Round-Robin Arbitrator
	----------------------------------------------------------------------------
	Signal jtag_grant_vec		: Std_Logic_Vector(1 downto 0) := (others => '0');
	Signal jtag_request_vec		: Std_Logic_Vector(1 downto 0) := (others => '0');
	signal jtag_state			: avs_state_type := idle_st;
	signal jtag_nxt_state		: avs_state_type := idle_st;
	----------------------------------------------------------------------------
	-- Signals for  Internal Memory Round-Robin Arbitrator
	----------------------------------------------------------------------------
	Signal int_mem_grant_vec	: Std_Logic_Vector(1 downto 0) := (others => '0');
	Signal int_mem_request_vec	: Std_Logic_Vector(1 downto 0) := (others => '0');
	signal int_mem_state		: avs_state_type := idle_st;
	signal int_mem_nxt_state	: avs_state_type := idle_st;
	----------------------------------------------------------------------------
	-- Signals for External Memory Round-Robin Arbitrator
	----------------------------------------------------------------------------
	Signal me_grant_vec			: Std_Logic_Vector(1 downto 0) := (others => '0');
	Signal me_request_vec		: Std_Logic_Vector(1 downto 0) := (others => '0');
	signal me_state				: avs_state_type := idle_st;
	signal me_nxt_state			: avs_state_type := idle_st;
	----------------------------------------------------------------------------
	-- Signals for Peripheral Unit Round-Robin Arbitrator
	----------------------------------------------------------------------------
	Signal io_grant_vec			: Std_Logic_Vector(1 downto 0) := (others => '0');
	Signal io_request_vec		: Std_Logic_Vector(1 downto 0) := (others => '0');
	signal io_state				: avs_state_type := idle_st;
	signal io_nxt_state			: avs_state_type := idle_st;
	----------------------------------------------------------------------------
	--  Signals for TimeBase Memory Unit Round-Robin Arbitrator
	----------------------------------------------------------------------------
	Signal time_grant_vec		: Std_Logic_Vector(1 downto 0) := (others => '0');
	Signal time_request_vec		: Std_Logic_Vector(1 downto 0) := (others => '0');
	signal time_state			: avs_state_type := idle_st;
	signal time_nxt_state		: avs_state_type := idle_st;
	----------------------------------------------------------------------------
Begin
------------------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    SubPart_MCU : ReplaceWith_MCU_CPU Port Map (
        Clk                => CLK_I,
        Reset_n            => Signal_Rst,
       

        I_ReadData         => Instr_ReadData,
        I_WaitRequest      => Instr_WaitRequest,
        I_Address          => Instr_Address,
        I_Read             => Instr_Read,
        I_ReadDataValid    => Instr_ReadDataValid,      -- IncludeIf_HasInstructionCache

        D_Address          => Data_Address,
        D_ByteEnable       => Data_ByteEnable,
        D_Read             => Data_Read,
        D_Write            => Data_Write,
        D_ReadData         => Data_ReadData,
        D_WriteData        => Data_WriteData,
        D_WaitRequest      => Data_WaitRequest,
        D_ReadDataValid    => Data_ReadDataValid,       -- IncludeIf_HasDataCache
		
		-- Processor Debug Interface
		vji_ir_out		=> JTAG_ir_out,						-- IncludeIf_OCDS
		vji_tdo			=> JTAG_tdo,						-- IncludeIf_OCDS
		vji_cdr			=> JTAG_cdr,						-- IncludeIf_OCDS
		vji_ir_in		=> JTAG_ir_in,						-- IncludeIf_OCDS
		vji_rti			=> JTAG_rti,						-- IncludeIf_OCDS
		vji_sdr     	=> JTAG_sdr,						-- IncludeIf_OCDS
		vji_tck     	=> JTAG_tck,						-- IncludeIf_OCDS
		vji_tdi     	=> JTAG_tdi,						-- IncludeIf_OCDS
		vji_udr     	=> JTAG_udr,						-- IncludeIf_OCDS
		vji_uir     	=> JTAG_uir,						-- IncludeIf_OCDS
			--no_ci_readra 
		jtag_debug_module_debugaccess_to_roms => Jtag_Debug_Module_DebugAccess_To_Roms,	-- IncludeIf_OCDS
		jtag_debug_module_resetrequest        => Jtag_Debug_Module_ResetRequest,		-- IncludeIf_OCDS
		jtag_debug_module_address             => Jtag_Debug_Module_Address,			-- IncludeIf_OCDS
		jtag_debug_module_byteenable          => Jtag_Debug_Module_ByteEnable,		-- IncludeIf_OCDS
		jtag_debug_module_debugaccess         => Jtag_Debug_Module_DebugAccess,		-- IncludeIf_OCDS
		jtag_debug_module_read                => Jtag_Debug_Module_Read,			-- IncludeIf_OCDS
		jtag_debug_module_readdata            => Jtag_Debug_Module_ReadData,		-- IncludeIf_OCDS
		jtag_debug_module_waitrequest         => jtag_Debug_Module_Waitrequest,		-- IncludeIf_OCDS
		jtag_debug_module_write               => Jtag_Debug_Module_Write,			-- IncludeIf_OCDS
		jtag_debug_module_writedata           => Jtag_Debug_Module_WriteData,		-- IncludeIf_OCDS
		
		D_Irq              => Interrupt_Cpu
    );
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
 	Combined_Rst				<= RST_I Or Jtag_Debug_Module_ResetRequest;
    Signal_Rst					<= Not Combined_Rst;
    IO_CLK_O					<= CLK_I;
    IO_RST_O					<= Combined_Rst;
    ME_CLK_O					<= CLK_I;
    ME_RST_O					<= Combined_Rst;
    TimeBaseReset				<= Combined_Rst;
    Interrupt_Cpu(0)			<= INT_I(0) or TimeBaseInterrupt;
	Interrupt_Cpu(31 Downto 1)	<= INT_I(31 Downto 1);
    ----------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------
	-- If Base Addresses have different lengths there should be full range address decoding !!!
	----------------------------------------------------------------------------------------------------------------------------
	-- Instruction Master Address Selection
	----------------------------------------------------------------------------------------------------------------------------
	-- Internal Memory Address
	InstrAddressIsInternalMemory	<='1'	when((unsigned(Instr_Address) >= INT_MEM_UNIT_BASE_ADDR)	and 		
												 (unsigned(Instr_Address) <= (INT_MEM_UNIT_BASE_ADDR+INT_MEM_UNIT_ADDR_LEN -1)))
											else '0';
	----------------------------------------------------------------------------------------------------------------------------
	-- Debug Unit Address
	InstrAddressIsDebugMemory		<='1'	when((unsigned(Instr_Address) >= DEBUG_UNIT_BASE_ADDR)  and 
												 (unsigned(Instr_Address) <=(DEBUG_UNIT_BASE_ADDR + DEBUG_UNIT_ADDR_LEN-1)))
											else '0';
	----------------------------------------------------------------------------------------------------------------------------
	-- External Memory Address
	InstrAddressIsExternalMemory	<='1'	when((unsigned(Instr_Address) >= EXT_MEM_UNIT_BASE_ADDR) and 
												 (unsigned(Instr_Address) <=(EXT_MEM_UNIT_BASE_ADDR+EXT_MEM_UNIT_ADDR_LEN -1)))
											else '0';
	----------------------------------------------------------------------------------------------------------------------------
	-- Peripheral Address 
	InstrAddressIsPeripheral		<= '1'	when(((B"0" & unsigned(Instr_Address)) >= PERIPHERAL_UNIT_BASE_ADDR)  and 
												 ((B"0" & unsigned(Instr_Address)) <=(PERIPHERAL_UNIT_BASE_ADDR+PERIPHERAL_UNIT_ADDR_LEN -1)))
											else '0';
	----------------------------------------------------------------------------------------------------------------------------
	-- TimeBase Address
	InstrAddressIsTimeBaseMemory	<='1'	when((B"0" & (unsigned(Instr_Address)) >= TIMEBASE_UNIT_BASE_ADDR) and 
												 (B"0" & (unsigned(Instr_Address)) <=(TIMEBASE_UNIT_BASE_ADDR+TIMEBASE_UNIT_ADDR_LEN -1)))
											else '0';
	----------------------------------------------------------------------------------------------------------------------------
	-- If Base Addresses have different lengths there should be full range address decoding !!!
	----------------------------------------------------------------------------------------------------------------------------
	-- Data Master Address Selection
	----------------------------------------------------------------------------------------------------------------------------
	-- Internal Memory Address
	DataAddressIsInternalMemory		<='1'	when((unsigned(Data_Address) >= INT_MEM_UNIT_BASE_ADDR)	and 		
												 (unsigned(Data_Address) <=(INT_MEM_UNIT_BASE_ADDR+INT_MEM_UNIT_ADDR_LEN -1)))
											else '0';
	-------------------------------------------------------------------------------------------------------------------------
	-- Debug Unit Address
	DataAddressIsDebugMemory		<='1'	when((unsigned(Data_Address) >= DEBUG_UNIT_BASE_ADDR)  and 
												 (unsigned(Data_Address) <=(DEBUG_UNIT_BASE_ADDR + DEBUG_UNIT_ADDR_LEN -1)))
											else '0';
	----------------------------------------------------------------------------------------------------------------------------
	-- External Memory Address
	DataAddressIsExternalMemory		<='1'	when((unsigned(Data_Address) >= EXT_MEM_UNIT_BASE_ADDR) and 
												 (unsigned(Data_Address) <=(EXT_MEM_UNIT_BASE_ADDR+EXT_MEM_UNIT_ADDR_LEN -1)))
											else '0'; 
	----------------------------------------------------------------------------------------------------------------------------
	-- TimeBase Address
	DataAddressIsPeripheral			<='1'	when(((B"0" & unsigned(Data_Address)) >= PERIPHERAL_UNIT_BASE_ADDR)  and 
												 ((B"0" & unsigned(Data_Address)) <=(PERIPHERAL_UNIT_BASE_ADDR+PERIPHERAL_UNIT_ADDR_LEN -1)))
											else '0'; 
	----------------------------------------------------------------------------------------------------------------------------
	-- TimeBase Address
	DataAddressIsTimeBaseMemory		<='1'	when(((B"0" & unsigned(Data_Address)) >= TIMEBASE_UNIT_BASE_ADDR) and 
												 ((B"0" & unsigned(Data_Address)) <= (TIMEBASE_UNIT_BASE_ADDR+TIMEBASE_UNIT_ADDR_LEN -1)))
											else '0';  
	----------------------------------------------------------------------------------------------------------------------------
	----------------------------------------------------------------------------
	-- Nios Instruction Master WaitRequest selection 
	----------------------------------------------------------------------------
	Instr_WaitRequest	<=	InternalMem_I_Wait	when (InstrAddressIsInternalMemory	= '1')	else
							DebugMem_I_Wait 	when (InstrAddressIsDebugMemory		= '1')	else
							TimeBaseMem_I_Wait	when (InstrAddressIsTimeBaseMemory	= '1')	else
							Peripheral_I_Wait	when (InstrAddressIsPeripheral		= '1')	else
							MainMemory_I_Wait	when (InstrAddressIsExternalMemory	= '1')	else
							'0';
	----------------------------------------------------------------------------
	-- Nios Instruction Master ReadDataValid selection 
	----------------------------------------------------------------------------
	Instr_ReadDataValid	<=	not InternalMem_I_Wait	when (Access_InternalMem_Instr	= '1')	else
							not DebugMem_I_Wait 	when (Access_DebugMem_Instr		= '1')	else
							not TimeBaseMem_I_Wait	when (Access_TimeBaseMem_Instr	= '1')	else
							not Peripheral_I_Wait	when (Access_IO_Instr			= '1')	else
							not MainMemory_I_Wait	when (Access_ME_Instr			= '1')	else
							'0';
	----------------------------------------------------------------------------
	-- Nios Instruction Master ReadData selection 
	----------------------------------------------------------------------------
	Instr_ReadData	<=  InstrRomIn					when (InstrAddressIsInternalMemory	= '1')	else
						Jtag_Debug_Module_ReadData 	when (InstrAddressIsDebugMemory		= '1')	else
						ME_DAT_I					when (InstrAddressIsExternalMemory	= '1')	else
						(others => '0');	
	----------------------------------------------------------------------------
	-- Nios Data Master WaitRequest selection 
	----------------------------------------------------------------------------
	Data_WaitRequest<=	InternalMem_D_Wait	when (DataAddressIsInternalMemory	= '1')	else
						DebugMem_D_Wait 	when (DataAddressIsDebugMemory 		= '1')	else
						TimeBaseMem_D_Wait	when (DataAddressIsTimeBaseMemory	= '1')	else
						Peripheral_D_Wait	when (DataAddressIsPeripheral		= '1')	else
						MainMemory_D_Wait	when (DataAddressIsExternalMemory	= '1')	else
						'0';
    ----------------------------------------------------------------------------
	-- Nios Data Master ReadData selection 
	----------------------------------------------------------------------------
	Data_ReadData	<=  DataRamIn					when (DataAddressIsInternalMemory = '1')	else
						Jtag_Debug_Module_ReadData 	when (DataAddressIsDebugMemory = '1')		else
						TimeBaseIn					when (DataAddressIsTimeBaseMemory = '1')	else
						IO_DAT_I					when ((DataAddressIsPeripheral = '1') and (Peripheral_Acked = '0')) else
						Peripheral_Data_In_Latch	when ((DataAddressIsPeripheral = '1') and (Peripheral_Acked = '1')) else
						ME_DAT_I					when (DataAddressIsExternalMemory = '1')	else
						(others => '0');
	----------------------------------------------------------------------------
	-- Nios Data Master ReadDataValid selection 
	----------------------------------------------------------------------------
	Data_ReadDataValid	<=	not InternalMem_D_Wait	when (Access_InternalMem_Data	= '1')	else	-- IncludeIf_HasDataCache
							not DebugMem_D_Wait 	when (Access_DebugMem_Data 		= '1')	else	-- IncludeIf_HasDataCache
							not TimeBaseMem_D_Wait	when (Access_TimeBaseMem_Data	= '1')	else	-- IncludeIf_HasDataCache
							not Peripheral_D_Wait	when (Access_IO_Data			= '1')	else	-- IncludeIf_HasDataCache
							not MainMemory_D_Wait	when (Access_ME_Data 			= '1')	else	-- IncludeIf_HasDataCache
							'0';																	-- IncludeIf_HasDataCache
	----------------------------------------------------------------------------
    Data_WriteData_Mux:
    Process(
			Data_WriteData,
			Data_ByteEnable
	)
    Begin
        EndianFixed_Data_ByteEnable <= Data_ByteEnable;
        EndianFixed_Data_WriteData  <= Data_WriteData;
    End Process;
    ----------------------------------------------------------------------------
	-- Jtag Debug Module Interface
	----------------------------------------------------------------------------
	Access_DebugMem_Data  <= (Data_Read  Or Data_Write) And DataAddressIsDebugMemory;
    Access_DebugMem_Instr <= (Instr_Read              ) And InstrAddressIsDebugMemory;
	----------------------------------------------------------------------------
	Jtag_Debug_Module_DebugAccess	<= Jtag_Debug_Module_DebugAccess_To_Roms;
	Jtag_Debug_Module_WriteData		<= Data_WriteData;
	master_selection_prc:
	process(
			jtag_grant_vec,
			Data_Address,
			Data_ByteEnable,
			Data_Write,
			Data_Read,
			Instr_Address,
			Instr_Read
	)
	begin
		if (jtag_grant_vec(data_mst) = '1') then				-- Data Master is Granted
			Jtag_Debug_Module_Address		<= Data_Address(10 Downto 2);
			Jtag_Debug_Module_ByteEnable	<= Data_ByteEnable;
			Jtag_Debug_Module_Write			<= Data_Write;
			Jtag_Debug_Module_Read			<= Data_Read;
		else													-- Instruction Master is Granted
			Jtag_Debug_Module_Address		<= Instr_Address(10 Downto 2);
			Jtag_Debug_Module_ByteEnable    <= "1111";
			Jtag_Debug_Module_Write         <= '0';
			Jtag_Debug_Module_Read			<= Instr_Read;
		end if;
	end process;
	----------------------------------------------------------------------------
	jtag_request_vec(instr_mst)	<= Access_DebugMem_Instr;	-- Assign Instruction Master Request
	jtag_request_vec(data_mst)	<= Access_DebugMem_Data;	-- Assign Data Master Request
	----------------------------------------------------------------------------
	-- JTAG Round-Robin Arbiter
	----------------------------------------------------------------------------
	jtag_rr_arbiter : ReplaceWith_Designator_nios_master_rr_arbiter 
					Port Map(
						CLK_I	=> CLK_I,
						RST_I	=> Combined_Rst,
						REQ_I	=> jtag_request_vec,
						GRANT_O	=> jtag_grant_vec
					);	
	----------------------------------------------------------------------------
	-- Debug Memory FSM
    ----------------------------------------------------------------------------
	jtag_fsm_prc:
	process(
			jtag_state,
			jtag_grant_vec,
			Access_DebugMem_Instr,
			Access_DebugMem_Data,
			Jtag_Debug_Module_Waitrequest
	)
	begin
		DebugMem_I_Wait	<= Access_DebugMem_Instr;
		DebugMem_D_Wait	<= Access_DebugMem_Data;
		case jtag_state is
		-- IDLE State	-- Waiting for Master Request
			when idle_st =>				
				if (Access_DebugMem_Instr = '1')  then
					DebugMem_I_Wait	<= '1';
					jtag_nxt_state	<= instr_access_st;
				elsif (Access_DebugMem_Data  = '1') then
					DebugMem_D_Wait	<= '1';
					jtag_nxt_state	<= data_access_st;
				else
					jtag_nxt_state	<= idle_st;
				end if;
		-- Instruction Access State
			when instr_access_st =>
				if (jtag_grant_vec(instr_mst) = '0') then
					DebugMem_I_Wait	<= '1';
					jtag_nxt_state	<= instr_access_st;
				Elsif (Jtag_Debug_Module_Waitrequest = '1') then
					DebugMem_I_Wait	<= '1';
					jtag_nxt_state	<= instr_access_st;
				else
					DebugMem_I_Wait	<= '0';
					jtag_nxt_state	<= idle_st;
				end if;	
		-- Data Access State
			when data_access_st =>
				if (jtag_grant_vec(data_mst) = '0') then
					DebugMem_D_Wait	<= '1';	
					jtag_nxt_state	<= data_access_st;
				Elsif (Jtag_Debug_Module_Waitrequest = '1')  then
					DebugMem_D_Wait	<= '1';	
					jtag_nxt_state	<= data_access_st;	
				else	
					DebugMem_D_Wait	<= '0';
					jtag_nxt_state	<= idle_st;
				end if;
		end case;
	end process;
	-- Jtag Debug Interface FSM Registers process
	jtag_fsm_prc_regs : process(CLK_I,Combined_Rst) is
	begin
		if(CLK_I'event and CLK_I = '1') then
			if (Combined_Rst = '1') then
				jtag_state <= idle_st;
			else
				jtag_state <= jtag_nxt_state;
			end if;
		end if;
	end process jtag_fsm_prc_regs;
    ----------------------------------------------------------------------------
	
    ----------------------------------------------------------------------------
    -- Internal Memory Interface;
    ----------------------------------------------------------------------------
    Access_InternalMem_Data  <= (Data_Read  Or Data_Write) And DataAddressIsInternalMemory;
    Access_InternalMem_Instr <= (Instr_Read              ) And InstrAddressIsInternalMemory;
    ----------------------------------------------------------------------------
	int_mem_request_vec(instr_mst)	<= Access_InternalMem_Instr;	-- Assign Instruction Master Request
	int_mem_request_vec(data_mst) 	<= Access_InternalMem_Data;		-- Assign Data Master Request
	----------------------------------------------------------------------------
	-- Internal Memory Round-Robin Arbitrator
	----------------------------------------------------------------------------
	intmem_rr_arbiter : ReplaceWith_Designator_nios_master_rr_arbiter 
							Port Map(
								CLK_I	=> CLK_I,
								RST_I	=> Combined_Rst,
								REQ_I	=> int_mem_request_vec,
								GRANT_O	=> int_mem_grant_vec
							);
	----------------------------------------------------------------------------
    InternalMemory_Control:
    Process (
			int_mem_grant_vec,
			EndianFixed_Data_WriteData,
			EndianFixed_Data_ByteEnable,
            DataAddressIsInternalMemory,
			Data_Address,
            Data_Write,
			Instr_Address
	)
    Begin
        DataRamOut   <= EndianFixed_Data_WriteData;
        DataRamBE    <= EndianFixed_Data_ByteEnable;
        DataRamAddr  <= Data_Address;
        InstrRomAddr <= Instr_Address;
	    DataRamWE <= '0';
		If (int_mem_grant_vec(data_mst) = '1') Then	-- Assign DataRamWE
			DataRamWE <= Data_Write;				-- When Data Master is granted
		End If;
    End Process InternalMemory_Control;
    ----------------------------------------------------------------------------
	-- Internal Memory FSM
	----------------------------------------------------------------------------
	InternalMemory_Combinational:
	process(
			int_mem_state,
			int_mem_grant_vec,
			Access_InternalMem_Instr,
			Access_InternalMem_Data
	)
	begin
		InternalMem_I_Wait	<= Access_InternalMem_Instr;
		InternalMem_D_Wait	<= Access_InternalMem_Data;
		case int_mem_state is
		-- IDLE State	-- Waiting for Master Request
			when idle_st =>
				if (Access_InternalMem_Instr = '1')  then
					InternalMem_I_Wait	<= '1';
					int_mem_nxt_state	<= instr_access_st;
				Elsif (Access_InternalMem_Data  = '1') then
					InternalMem_D_Wait	<= '1';
					int_mem_nxt_state	<= data_access_st;
				else
					int_mem_nxt_state	<= idle_st;
				end if;
		-- Instruction Access State 
			when instr_access_st =>
				 if (int_mem_grant_vec(instr_mst) = '0') then	-- Waiting for Grant Signal 
					 InternalMem_I_Wait	<= '1';			
					 int_mem_nxt_state	<= instr_access_st;
				 else											-- Instruction Master is Granted 
					InternalMem_I_Wait	<= '0';
					int_mem_nxt_state	<= idle_st;
				end if;	
		-- Data Access State
			when data_access_st =>						
				if (int_mem_grant_vec(data_mst) = '0') then		-- Waiting for Grant Signal
					InternalMem_D_Wait	<= '1';	
					int_mem_nxt_state	<= data_access_st;
				else											-- Data Master is Granted 
					InternalMem_D_Wait	<= '0';
					int_mem_nxt_state	<= idle_st;
				end if;
		end case;
	end process InternalMemory_Combinational;
	----------------------------------------------------------------------------
	int_mem_fsm_prc_regs : process(CLK_I, Combined_Rst) is
	begin
		if(CLK_I'event and CLK_I = '1') then
			if (Combined_Rst = '1') then
				int_mem_state <= idle_st;
			else	
				int_mem_state <= int_mem_nxt_state;
			end if;	
		end if;
	end process int_mem_fsm_prc_regs;
    ----------------------------------------------------------------------------
    -- 1 clock wait for internal mem (remove in the future, once we have tightly coupled mem. do this by setting InternalMem_Running to 0 permanently)
    -- RAF: shouldn't InternalMem_Running be set in InternalMem_Idle whichever interface is requesting access and released in InternalMem_Wait? (need to check that with avalon spec)
    --      Also, data and instruction interfaces are independed (dualport ram that we use) so we need a separate decoder for instruction and data sides.
    --      According to avalon spec, waitrequest must be asserted within the first clock cycle of transfer (avalon spec p.34), but be carefull here, check if this works.
    ----------------------------------------------------------------------------
 
    ----------------------------------------------------------------------------
    -- Wishbone IO Peripheral Interface;
    ----------------------------------------------------------------------------
    Access_IO_Data  <= (Data_Read  Or Data_Write) And DataAddressIsPeripheral;
    Access_IO_Instr <= (Instr_Read              ) And InstrAddressIsPeripheral;
	----------------------------------------------------------------------------
	io_request_vec(instr_mst)	<= Access_IO_Instr;	-- Assign Instruction Master Request
	io_request_vec(data_mst) 	<= Access_IO_Data;	-- Assign Data Master Request
	----------------------------------------------------------------------------
	-- IO Interface Round-Robin Arbiter 
	----------------------------------------------------------------------------
	io_rr_arbiter : ReplaceWith_Designator_nios_master_rr_arbiter 	Port Map(
												CLK_I	=> CLK_I,
												RST_I	=> Combined_Rst,
												REQ_I	=> io_request_vec,
												GRANT_O	=> io_grant_vec
											);		
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    WishboneIO_Acked:
    Process(
			CLK_I,
			Peripheral_Running,
			IO_ACK_I,
			IO_DAT_I
	)
	Begin
		If rising_edge(CLK_I) Then
			If Peripheral_Running = '1' Then
				If IO_ACK_I = '1' Then
					Peripheral_Acked <= '1';
					-- remember the current output data from the peripheral,
					-- now that the peripheral acked we will remove the strobe
					-- so the data on IO_DAT_I will not be valid anymore
					Peripheral_Data_In_Latch <= IO_DAT_I;
				End If;
			Else
				Peripheral_Acked <= '0';
			End If;
		End If;
	End Process;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    WishboneIO_Outputs:
    Process( 
			Peripheral_Running,
			Data_Address,
			EndianFixed_Data_ByteEnable,
			Data_Write,
			EndianFixed_Data_WriteData,
			Instr_Address,
			Peripheral_Acked
	)
	Begin
		If Peripheral_Acked = '0' Then
			IO_STB_O <= Peripheral_Running;
			IO_CYC_O <= Peripheral_Running;
		Else
			IO_STB_O <= '0';
			IO_CYC_O <= '0';
		End If;

		IO_DAT_O <= EndianFixed_Data_WriteData;

		If (io_grant_vec(data_mst) = '1') Then		-- Data Master is granted
			IO_ADR_O <= Data_Address(23 Downto 0);	
			IO_SEL_O <= EndianFixed_Data_ByteEnable;
			IO_WE_O  <= Data_Write;
		Else								-- Else Instruction Master is granted
			IO_ADR_O <= Instr_Address(23 Downto 0);
			IO_SEL_O <= "1111";
			IO_WE_O  <= '0';
		End If;
	End Process;
    ----------------------------------------------------------------------------
	-- Peripheral Unit FSM
    ----------------------------------------------------------------------------
	WishboneFSM_IO_Combinatorial:
    Process(
			io_state,
			io_grant_vec,
			Access_IO_Instr,
			Access_IO_Data,
			IO_ACK_I,
			Peripheral_Acked,
			TimeOut_IO
	)
    Begin
        Peripheral_I_Wait  <= Access_IO_Instr;
        Peripheral_D_Wait  <= Access_IO_Data;
        Peripheral_Running <= Access_IO_Instr Or Access_IO_Data;
        Case io_state Is
		-- IDLE State	-- Waiting for Master Request	
			when idle_st =>				
				if (Access_IO_Instr = '1')  then
					Peripheral_I_Wait	<= '1';
					io_nxt_state		<= instr_access_st;
				elsif (Access_IO_Data  = '1') then
					Peripheral_D_Wait	<= '1';
					io_nxt_state		<= data_access_st;
				else
					io_nxt_state		<= idle_st;
				end if;
		-- Instruction Access State
			when instr_access_st =>
				if (io_grant_vec(instr_mst) = '0') then
					Peripheral_I_Wait	<= '1';
					io_nxt_state		<= instr_access_st;
				else
					Peripheral_I_Wait	<= '1';
					Peripheral_Running	<= '1';
					If (IO_ACK_I = '1') Or (Peripheral_Acked = '1') Or (TimeOut_IO = '1') Then
						Peripheral_I_Wait	<= '0';
						io_nxt_state		<= idle_st;
					else
						io_nxt_state		<= instr_access_st;
					End If;
				end if;	
		-- Data Access State
			when data_access_st =>
				if (io_grant_vec(data_mst) = '0') then
					Peripheral_D_Wait	<= '1';	
					io_nxt_state		<= data_access_st;
				else	
					Peripheral_D_Wait	<= '1';
					Peripheral_Running	<= '1';
					If (IO_ACK_I = '1') Or (Peripheral_Acked = '1') Or (TimeOut_IO = '1') Then
						Peripheral_D_Wait	<= '0';
						io_nxt_state		<= idle_st;
					else
						io_nxt_state		<= data_access_st;
					End If;
				end if;
        End Case;
    End Process;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    io_fsm_prc_regs : process(CLK_I, Combined_Rst) is
	begin
		if(CLK_I'event and CLK_I = '1') then
			if (Combined_Rst = '1') then
				io_state 	<= idle_st;
			else
				io_state 	<= io_nxt_state;
			end if;	
		end if;
	end process io_fsm_prc_regs;
	----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    WishboneFSM_IO_State_Decoder:
    Process ( io_state, io_nxt_state )
    Begin
        Case io_state Is
            When idle_st =>
                Debug_State_IO <= "00";
            When instr_access_st =>
                Debug_State_IO <= "01";
            When data_access_st =>
                Debug_State_IO <= "10";
            When Others =>
                Debug_State_IO <= "11";
        End Case;

        Case io_nxt_state Is
            When idle_st =>
                Debug_StateNext_IO <= "00";
            When instr_access_st =>
                Debug_StateNext_IO <= "01";
            When data_access_st =>
                Debug_StateNext_IO <= "10";
            When Others =>
                Debug_StateNext_IO <= "11";
        End Case;
    End Process WishboneFSM_IO_State_Decoder;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    -- Wishbone Memory Interface;
    ----------------------------------------------------------------------------
    Access_ME_Data  <= (Data_Read  Or Data_Write) And DataAddressIsExternalMemory;
    Access_ME_Instr <= (Instr_Read              ) And InstrAddressIsExternalMemory;
    Access_ME       <= Access_ME_Data Or Access_ME_Instr;
    ----------------------------------------------------------------------------
	me_request_vec(instr_mst)	<= Access_ME_Instr; -- Assign Instruction Master Request
	me_request_vec(data_mst) 	<= Access_ME_Data;	-- Assign Data Master Request
	----------------------------------------------------------------------------
	-- External Memory Interface Round-Robin Arbitrator 
	----------------------------------------------------------------------------
	me_rr_arbiter : ReplaceWith_Designator_nios_master_rr_arbiter Port Map(
												CLK_I	=> CLK_I,
												RST_I	=> Combined_Rst,
												REQ_I	=> me_request_vec,
												GRANT_O	=> me_grant_vec
											);		
    ----------------------------------------------------------------------------
	WishboneME_Outputs:
	Process(
			me_grant_vec,
			MainMemory_Running,
			Data_Address,
			Access_ME_Data,
			EndianFixed_Data_ByteEnable,
			Data_Write,
			EndianFixed_Data_WriteData,
			Instr_Address
	)
	Begin
		ME_STB_O <= MainMemory_Running;
		ME_CYC_O <= MainMemory_Running;
		ME_DAT_O <= EndianFixed_Data_WriteData;

		If (me_grant_vec(data_mst) = '1') Then	-- Data Master is Granted
			ME_ADR_O <= Data_Address;
			ME_SEL_O <= EndianFixed_Data_ByteEnable;
			ME_WE_O  <= Data_Write;
		Else							-- Instruction Master is Granted
			ME_ADR_O <= Instr_Address;
			ME_SEL_O <= "1111";
			ME_WE_O  <= '0';
		End If;
	End Process WishboneME_Outputs;
    ----------------------------------------------------------------------------
	-- External Memory FSM
	----------------------------------------------------------------------------
    WishboneFSM_ME_Combinatorial:
    Process(
			me_state,
			me_grant_vec,
			Access_ME_Instr,
			Access_ME_Data,
			ME_ACK_I,
			TimeOut_ME
	)
	Begin
        MainMemory_I_Wait  <= Access_ME_Instr;
        MainMemory_D_Wait  <= Access_ME_Data;
        MainMemory_Running <= Access_ME_Instr Or Access_ME_Data;
        Case me_state Is
		-- IDLE State	-- Waiting for Master Request	
			when idle_st =>				
				if (Access_ME_Instr = '1')  then		-- Instruction Master Sent Request
					MainMemory_I_Wait	<= '1';
					me_nxt_state		<= instr_access_st;
				elsif (Access_ME_Data  = '1') then
					MainMemory_D_Wait 	<= '1';			-- Data Master Sent Request
					me_nxt_state		<= data_access_st;
				else
					me_nxt_state		<= idle_st;
				end if;
		-- Instruction Access State
			when instr_access_st =>
				if (me_grant_vec(instr_mst) = '0') then		-- Waiting for Grant signal
					MainMemory_I_Wait	<= '1';				
					me_nxt_state		<= instr_access_st;
				else										-- Master is Granted
					MainMemory_I_Wait  <= '1';
					MainMemory_Running <= '1';
					If (ME_ACK_I = '1') Or (TimeOut_ME = '1') Then -- Waiting for slave
						MainMemory_I_Wait	<= '0';
						me_nxt_state		<= idle_st;
					else
						me_nxt_state		<= instr_access_st;
					End If;
				end if;	
		-- Data Access State
			when data_access_st =>
				if (me_grant_vec(data_mst) = '0') then		-- Waiting for Grant signal
					MainMemory_D_Wait	<= '1';	
					me_nxt_state		<= data_access_st;
				else										-- Master is not Granted
					MainMemory_D_Wait  <= '1';
					MainMemory_Running <= '1';
					If (ME_ACK_I = '1') Or (TimeOut_ME = '1') Then -- Waiting for slave
						MainMemory_D_Wait <= '0';
						me_nxt_state	<= idle_st;
					else
						me_nxt_state	<= data_access_st;
					End If;
				end if;
        End Case;
    End Process WishboneFSM_ME_Combinatorial;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
	me_fsm_prc_regs : process(CLK_I, Combined_Rst) is
	begin
		if(CLK_I'event and CLK_I = '1') then
			if (Combined_Rst = '1') then
				me_state 	<= idle_st;
			else
				me_state 	<= me_nxt_state;
			end if;	
		end if;
	end process me_fsm_prc_regs;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    -- TimeBase Memory Interface;
    ----------------------------------------------------------------------------
    Access_TimeBaseMem_Data  <= (Data_Read  Or Data_Write) And DataAddressIsTimeBaseMemory;
    Access_TimeBaseMem_Instr <= (Instr_Read              ) And InstrAddressIsTimeBaseMemory;
    ----------------------------------------------------------------------------
	----------------------------------------------------------------------------
	time_request_vec(instr_mst)	<= Access_TimeBaseMem_Instr;	-- Assign Instruction Master Request 
	time_request_vec(data_mst) 	<= Access_TimeBaseMem_Data;	-- Assign Data Master Request
	----------------------------------------------------------------------------
	--TimeBase Interface Round-Robin Arbitrator
	----------------------------------------------------------------------------
	timeme_rr_arbiter : ReplaceWith_Designator_nios_master_rr_arbiter 
					Port Map(
							CLK_I	=> CLK_I,
							RST_I	=> Combined_Rst,
							REQ_I	=> time_request_vec,
							GRANT_O	=> time_grant_vec
					);
    ----------------------------------------------------------------------------
    TimeBaseMemory_Control:
    Process(
			time_grant_vec,
			Data_WriteData,
            Data_Address,
            Data_Write,
            Instr_Address
	)
    Begin
        TimeBaseOut <= Data_WriteData;
        If (time_grant_vec(data_mst) = '1') Then	-- Data Master is granted
            TimeBaseAddr <= Data_Address(3 Downto 2);
            TimeBaseWE   <= Data_Write;
        Else										-- Instruction Master is granted
            TimeBaseAddr <= Instr_Address(3 Downto 2);
            TimeBaseWE   <= '0';		
        End If;
    End Process;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    TimeBaseMemory_Combinational:
    Process(
			time_state,
			time_grant_vec,
			Access_TimeBaseMem_Instr,
			Access_TimeBaseMem_Data
	)
	Begin
        TimeBaseMem_I_Wait   <= Access_TimeBaseMem_Instr;
        TimeBaseMem_D_Wait   <= Access_TimeBaseMem_Data;
        Case time_state Is
		-- IDLE State	
			when idle_st =>				
				if (Access_TimeBaseMem_Instr = '1')  then
					TimeBaseMem_I_Wait  <= '1';
					time_nxt_state		<= instr_access_st;
				elsif (Access_TimeBaseMem_Data  = '1') then
					TimeBaseMem_D_Wait  <= '1';
					time_nxt_state		<= data_access_st;
				else
					time_nxt_state		<= idle_st;
				end if;
		-- Instruction Access State
			when instr_access_st =>
				if (time_grant_vec(instr_mst) = '0') then	-- Waiting for Grant signal			
					TimeBaseMem_I_Wait	<= '1';				
					time_nxt_state		<= instr_access_st;
				else										-- Master is Granted
					TimeBaseMem_I_Wait  <= '0';
					time_nxt_state		<= idle_st;
				end if;	
		-- Data Access State
			when data_access_st =>
				if (time_grant_vec(data_mst) = '0') then	-- Waiting for Grant signal		
					TimeBaseMem_D_Wait	<= '1';				
					time_nxt_state		<= data_access_st;
				else										-- Master is Granted
					TimeBaseMem_D_Wait  <= '0';
					time_nxt_state		<= idle_st;
				end if;
        End Case;
    End Process TimeBaseMemory_Combinational;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
	timebase_fsm_prc_regs : process(CLK_I, Combined_Rst) is
	begin
		if(CLK_I'event and CLK_I = '1') then
			if (Combined_Rst = '1') then
				time_state 	<= idle_st;
			else
				time_state	<= time_nxt_state;
			end if;		
		end if;
	end process timebase_fsm_prc_regs;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    TimeOutCounter_IO :
    Process(
			CLK_I,
            TimeOut_IO_Counter,
			Combined_Rst,
			Peripheral_I_Wait,
			Peripheral_D_Wait
	)
    Begin
        If Rising_Edge(CLK_I) Then
            If (Combined_Rst = '1') Or ((Peripheral_I_Wait = '0') And (Peripheral_D_Wait = '0')) Then
                TimeOut_IO_Counter <= (Others=>'0') ;
            Else
                TimeOut_IO_Counter <= TimeOut_IO_Counter + 1 ;
            End If ;
        End if ;

        TimeOut_IO <= TimeOut_IO_Counter(12) ;
    End Process TimeOutCounter_IO;
    ----------------------------------------------------------------------------
    ----------------------------------------------------------------------------
    TimeOutCounter_ME:
    Process(
			CLK_I,
            Combined_Rst,
			MainMemory_I_Wait,
			MainMemory_D_Wait,
			TimeOut_ME_Counter
	)
    Begin
        If Rising_Edge(CLK_I) Then
            If (Combined_Rst = '1') Or ((MainMemory_I_Wait = '0') And (MainMemory_D_Wait = '0')) Then
                TimeOut_ME_Counter <= (Others=>'0') ;
            Else
                TimeOut_ME_Counter <= TimeOut_ME_Counter + 1 ;
            End If ;
        End if ;

        TimeOut_ME <= TimeOut_ME_Counter(12) ;
    End Process TimeOutCounter_ME;
    ----------------------------------------------------------------------------
	----------------------------------------------------------------------------
    LAX_CPU_I_ReadData      <= Instr_ReadData;
    LAX_CPU_I_WaitRequest   <= Instr_WaitRequest;
    LAX_CPU_I_Address       <= Instr_Address;
    LAX_CPU_I_Read          <= Instr_Read;
    LAX_CPU_I_ReadDataValid <= Instr_ReadDataValid;
    LAX_CPU_D_Address       <= Data_Address;
    LAX_CPU_D_ByteEnable    <= Data_ByteEnable;
    LAX_CPU_D_Read          <= Data_Read;
    LAX_CPU_D_Write         <= Data_Write;
    LAX_CPU_D_ReadData      <= Data_ReadData;
    LAX_CPU_D_WriteData     <= Data_WriteData;
    LAX_CPU_D_WaitRequest   <= Data_WaitRequest;
	LAX_CPU_D_ReadDataValid <= Data_ReadDataValid; -- IncludeIf_HasDataCache
	
	-- Virtual JTAG interface	
	LAX_jtag_debug_module_debugaccess_to_roms	<= jtag_debug_module_debugaccess_to_roms;	-- IncludeIf_OCDS
	LAX_jtag_debug_module_resetrequest			<= jtag_debug_module_resetrequest;			-- IncludeIf_OCDS
	LAX_jtag_debug_module_debugaccess			<= jtag_debug_module_debugaccess;			-- IncludeIf_OCDS
	LAX_jtag_debug_module_read					<= jtag_debug_module_read;					-- IncludeIf_OCDS
	LAX_jtag_debug_module_waitrequest			<= jtag_debug_module_waitrequest;			-- IncludeIf_OCDS
	LAX_jtag_debug_module_write					<= jtag_debug_module_write;					-- IncludeIf_OCDS
	
	
	Debug_MainMemory_I_Wait  <= MainMemory_I_Wait;
    Debug_MainMemory_D_Wait  <= MainMemory_D_Wait;
    Debug_Peripheral_I_Wait  <= Peripheral_I_Wait;
    Debug_Peripheral_D_Wait  <= Peripheral_D_Wait;
    Debug_InternalMem_I_Wait <= InternalMem_I_Wait;
    Debug_InternalMem_D_Wait <= InternalMem_D_Wait;
    Debug_DebugMem_I_Wait    <= DebugMem_I_Wait;
    Debug_DebugMem_D_Wait    <= DebugMem_D_Wait;
	Debug_TimeBase_I_Wait    <= TimeBaseMem_D_Wait;
    Debug_TimeBase_D_Wait    <= TimeBaseMem_I_Wait;
	
	Debug_Access_ME_Data			<= Access_ME_Data;
	Debug_Access_ME_Instr			<= Access_ME_Instr;
	Debug_Access_ME      			<= Access_ME;
	Debug_Access_IO_Data 			<= Access_IO_Data;
	Debug_Access_IO_Instr			<= Access_IO_Instr;
	Debug_Access_InternalMem_Data	<= Access_InternalMem_Data;
	Debug_Access_InternalMem_Instr	<= Access_InternalMem_Instr;
	Debug_Access_DebugMem_Data    	<= Access_DebugMem_Data;
	Debug_Access_DebugMem_Instr   	<= Access_DebugMem_Instr;
	Debug_Access_TimeBaseMem_Data 	<= Access_TimeBaseMem_Data;
	Debug_Access_TimeBaseMem_Instr	<= Access_TimeBaseMem_Instr;
	
	-- For Debugging
	Debug_InstrAddressIsInternalMemory	<= InstrAddressIsInternalMemory;
	Debug_InstrAddressIsPeripheral     <= InstrAddressIsPeripheral;
	Debug_InstrAddressIsExternalMemory <= InstrAddressIsExternalMemory;
	Debug_InstrAddressIsDebugMemory    <= InstrAddressIsDebugMemory;
	Debug_InstrAddressIsTimeBaseMemory <= InstrAddressIsTimeBaseMemory;
	Debug_DataAddressIsInternalMemory  <= DataAddressIsInternalMemory;
	Debug_DataAddressIsPeripheral      <= DataAddressIsPeripheral;
	Debug_DataAddressIsExternalMemory  <= DataAddressIsExternalMemory;
	Debug_DataAddressIsDebugMemory     <= DataAddressIsDebugMemory;
	Debug_DataAddressIsTimeBaseMemory  <= DataAddressIsTimeBaseMemory;
------------------------------------------------------------------------------------
End RTL;
------------------------------------------------------------------------------------



