package require cmdline
variable ::argv0 $::quartus(args)
puts "*************************************************************************"
puts "********************* Script Arguments **********************************"
puts "********************* Quartus Version 13.0 ******************************"

# Assign Arguments
set options {
	{ "tool_version.arg"  "" "Version" } \
	{ "quartus_dir.arg"   "" "Quartus Directory" } \
	{ "cache_dir.arg"     "" "Cache Folder" } \
	{ "nios2kit_dir.arg"  "" "NIOS 2 KIT Folder" } \
	{ "proj_out_dir.arg"  "" "Projecr output Folder" } \
}

array set optshash [::cmdline::getoptions ::argv0 $options]

set systemTime [clock seconds]
puts "The time is: [clock format $systemTime -format %H:%M:%S]"
puts "The date is: [clock format $systemTime -format %D]"

set TOOL_VER             "$optshash(tool_version)"
set QUARTUS_DIR          "$optshash(quartus_dir)"
set CACHE_FOLDER         "$optshash(cache_dir)"
set NIOS2_KIT            "$optshash(nios2kit_dir)"
set PROJECT_OUT_FOLDER	 "$optshash(proj_out_dir)"
set DEVICE               "$(DEVICE)"
set FAMILY               "$(FAMILY)"
set CPU_NAME             "$(CPU_NAME)"
set CPU_SELECTION        "$(CPU_SELECTION)"
set CPU_IMPLEMENTATION   "$(CPU_IMPLEMENTATION)"
set JTAG_MODULE_ENABLE   "$(JTAG_MODULE_ENABLE)" 	
set BREAKPOINT_ON_HREset "$(BREAKPOINT_ON_HREset)"
set HARDWARE_MDU         "$(HARDWARE_MDU)" 
puts "Device                     is $DEVICE"
puts "Device Family              is $FAMILY"
puts "Processor name             is $CPU_NAME"
puts "Processor selection        is $CPU_SELECTION"
puts "Processor implementation   is $CPU_IMPLEMENTATION"
puts "Projecr output Folder      is $PROJECT_OUT_FOLDER"

# Assign path
set search_path "$CACHE_FOLDER\,\$"
puts "Search folder             is $search_path"
set QSYS_BIN_DIR "$QUARTUS_DIR/sopc_builder/bin"
puts "QSYS_BIN_DIR              is $QSYS_BIN_DIR"
set QSYS_FILE_NAME "$CACHE_FOLDER/$CPU_NAME.qsys"
puts "QSYS_FILE_NAME            is $QSYS_FILE_NAME"
set SCRIPT_NAME "$CACHE_FOLDER/qsys_gen_script.tcl"
puts "SCRIPT_NAME               is $SCRIPT_NAME"


set qsys_synth_path "$CACHE_FOLDER/$CPU_NAME/synthesis"
puts "qsys_synth_path           is $qsys_synth_path"
set qsys_modules_path "$CACHE_FOLDER/$CPU_NAME/synthesis/submodules"
puts "qsys_modules_path         is $qsys_modules_path"

# Create System
puts "*************** RUN QSYS-SCRIPT ***************************************************"
if	{[catch {exec "$QSYS_BIN_DIR/qsys-script.exe" "--search-path=$search_path"\
												  "--script=$SCRIPT_NAME"\
	     	} script_msg]\
    } then {
	puts "Something seems to have gone wrong:"
	puts "Information about it:"
	puts $script_msg
} else {
	puts "*************** END QSYS-SCRIPT  **************************************************"
}

puts "*************** END QSYS-SCRIPT  **************************************************"
# Generate System
puts "*************** RUN QSYS-GENERATE *************************************************"
if	{[catch {exec "$QSYS_BIN_DIR/qsys-generate.exe" "$QSYS_FILE_NAME" "--synthesis=VHDL"\
                                                                      "--search-path=$search_path"\
																	  "--family=$FAMILY"\
																	  "--part=$DEVICE"\
																	  "--block-symbol-file"\
			} generate_msg]\
	} then {
		puts "Something seems to have gone wrong:"
		puts "Information about it:"
		puts $generate_msg
} else {
	puts "*************** QSYS-GENERATE OK ************************************************"
}
puts "*************** END QSYS-GENERATE**************************************************"
cd $qsys_synth_path
set filenames1 [glob -nocomplain * .*]
foreach filename $filenames1 {
    if	{ [file isfile $filename]
		} then {
			if { [ string match "*.vhd" $filename] ||
				 [ string match "*.v" $filename] ||
				 [ string match "*.sv" $filename] ||
				 [ string match "*.cmp" $filename] ||
				 [ string match "*.SDC" $filename] ||
				 [ string match "*.sdc" $filename] ||
				 [ string match "*.qip" $filename]
			} then {
				puts "Copy file $filename"
				file copy -force $filename $PROJECT_OUT_FOLDER
			}	
	}
}
cd $qsys_modules_path
set filenames2 [glob -nocomplain * .*]
 foreach filename $filenames2 {
    if	{ [file isfile $filename]
		} then {
			if { [ string match "*.vhd" $filename] ||
				 [ string match "*.v" $filename] ||
				 [ string match "*.sv" $filename] ||
				 [ string match "*.cmp" $filename] ||
				 [ string match "*.SDC" $filename] ||
				 [ string match "*.sdc" $filename] ||
				 [ string match "*.qip" $filename]
			} then {
				puts "Copy file $filename"
				file copy -force $filename $PROJECT_OUT_FOLDER
			}	
	}
}	
if {
	$JTAG_MODULE_ENABLE == "Enable"
} then {
	# ********************************************************************** 
	set		ociram_default_file ""
	append	ociram_default_file $CACHE_FOLDER "/cpu_ociram_default_contents.mif"
	puts	"ociram_default_file  $ociram_default_file"

	set		ociram_out_file ""
	append	ociram_out_file $qsys_modules_path "/" $CPU_NAME "_CPU_ociram_default_contents.mif"
	puts	"ociram_out_file  $ociram_out_file"

	puts "Delete gen file"
	#if {[file exist $ociram_out_file]}{
	#	file delete -force $ociram_out_file
	#}
	puts "Copy default RAM contents to gen file"
	set ociram_default_id [open $ociram_default_file r]
	set ociram_out_id     [open $ociram_out_file w]

	puts -nonewline $ociram_out_id [read $ociram_default_id]
	close $ociram_out_id
	close $ociram_default_id
}


unset options
unset systemTime
unset TOOL_VER
unset QUARTUS_DIR
unset CACHE_FOLDER
unset NIOS2_KIT
unset DEVICE
unset FAMILY
unset CPU_NAME
unset CPU_SELECTION
unset CPU_IMPLEMENTATION
unset PROJECT_OUT_FOLDER
unset JTAG_MODULE_ENABLE
unset BREAKPOINT_ON_HREset
unset HARDWARE_MDU
unset search_path
unset QSYS_BIN_DIR
unset QSYS_FILE_NAME
unset SCRIPT_NAME
unset qsys_synth_path
unset qsys_modules_path
unset filenames1
unset filenames2
 

